/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.doc;

import freemarker.template.*;

/**
 * Encapsulates a single table-of-contents entry in the FreeMarker manual.
 *
 * @version $Id: ContentEntry.java,v 1.8 2003/11/08 00:11:59 run2000 Exp $
 */
public final class ContentEntry implements TemplateHashModel, TemplateScalarModel {

    /** Holds value of property title. */
    private String m_aTitle;

    /** Holds value of property longTitle. */
    private String m_aLongTitle;

    /** Holds value of property link. */
    private String m_aLink;

    /** Holds value of property superTitle */
    private String m_aSuperTitle;

    /** Holds value of property generated. */
    private boolean m_bIsGenerated;

    /** Holds value of property generated. */
    private String m_aDescription;

    /** Creates new ContentEntry */
    public ContentEntry() {
        m_bIsGenerated = true;
    }

    /** Creates new ContentEntry with supplied title and link */
    public ContentEntry( String title, String link ) {
        m_aTitle = title;
        m_aLongTitle = title;
        m_aLink = link;
        m_bIsGenerated = true;
    }

    /** Creates new ContentEntry with supplied title, long title and link */
    public ContentEntry( String title, String link, String longTitle ) {
        m_aTitle = title;
        m_aLongTitle = longTitle;
        m_aLink = link;
        m_bIsGenerated = true;
    }

    /**
     * Gets a {@link freemarker.template.TemplateModel} from the hash.
     *
     * @param key the name by which the <code>TemplateModel</code>
     * is identified in the template.
     * @return the <code>TemplateModel</code> referred to by the key,
     * or <code>null</code> if not found.
     */
    public TemplateModel get(String key) throws TemplateModelException {
        if( key.equals( "title" )) {
            return new FastScalar( m_aTitle );
        } else if( key.equals( "longTitle" )) {
            return new FastScalar( m_aLongTitle );
        } else if( key.equals( "link" )) {
            return new FastScalar( m_aLink );
        } else if( key.equals( "generate" )) {
            return FastBoolean.getInstance( m_bIsGenerated );
        } else if( key.equals( "superTitle" )) {
            return new FastScalar( m_aSuperTitle );
        } else if( key.equals( "description" )) {
            return new FastScalar( m_aDescription );
        } else {
            return null;
        }
    }

    /**
     * Returns the title of this content entry as a String.
     *
     * @return the title of this content entry.
     */
    public String getAsString() throws TemplateModelException {
        return m_aTitle;
    }

    /**
     * Is the template model empty?
     *
     * @return <code>false</code> to indicate the object is never empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }

    /** Getter for property title.
     * @return Value of property title.
     */
    public String getTitle() {
        return m_aTitle;
    }

    /** Setter for property title.
     * @param title New value of property title.
     */
    public void setTitle(String title) {
        m_aTitle = title;
    }

    /** Getter for property longTitle.
     * @return Value of property longTitle.
     */
    public String getLongTitle() {
        return m_aLongTitle;
    }

    /** Setter for property longTitle.
     * @param longTitle New value of property longTitle.
     */
    public void setLongTitle(String longTitle) {
        m_aLongTitle = longTitle;
    }

    /** Getter for property link.
     * @return Value of property link.
     */
    public String getLink() {
        return m_aLink;
    }

    /** Setter for property link.
     * @param link New value of property link.
     */
    public void setLink(String link) {
        m_aLink = link;
    }

    /** Getter for property generated.
     * @return Value of property generated.
     */
    public boolean isGenerated() {
        return m_bIsGenerated;
    }

    /** Setter for property generated.
     * @param generated New value of property generated.
     */
    public void setGenerated(boolean generated) {
        m_bIsGenerated = generated;
    }

    /** Getter for property superTitle.
     * @return Value of property superTitle.
     */
    public String getSuperTitle() {
        return m_aSuperTitle;
    }

    /** Setter for property superTitle.
     * @param superTitle New value of property superTitle.
     */
    public void setSuperTitle(String superTitle) {
        m_aSuperTitle = superTitle;
    }

    /** Getter for property description.
     * @return Value of property description.
     */
    public String getDescription() {
        return m_aDescription;
    }

    /** Setter for property description.
     * @param description New value of property description.
     */
    public void setDescription(String description) {
        m_aDescription = description;
    }
}
