/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.doc;

import java.io.*;
import java.util.Map;
import java.util.Properties;
import java.util.HashMap;

import org.xml.sax.XMLReader;
import freemarker.template.*;
import freemarker.ext.misc.FreeMarkerToHtml;
import freemarker.ext.misc.JavaToHtml;
import freemarker.ext.misc.CompressWhitespace;
import freemarker.ext.jdom.NodeListModel;
import org.jdom.*;
import org.jdom.input.SAXBuilder;

/**
 * Creates the FM-Classic manual.
 *
 * @version $Id: Manual.java,v 1.20 2003/11/28 01:21:44 run2000 Exp $
 */
public final class Manual {
    private ContentList m_cContents;
    private Map m_cProperties;

    /** Creates new Manual */
    public Manual() {
    }

    /**
     * Parses the <code>contents.xml</code> file using a simple SAX parser,
     * and retrieves the resulting {@link ContentList}.
     */
    private void parseContents() throws Exception {
        File cContentFile;

        cContentFile = new File( getRootPath(), "src/freemarker/doc/xml/contents.xml" );
        String aLocation = cContentFile.toURL().toString();
        ContentsBuilder cBuilder = new ContentsBuilder();
        XMLReader cParser = new org.apache.xerces.parsers.SAXParser();

        cParser.setContentHandler( cBuilder );
        cParser.setErrorHandler( cBuilder );
        cParser.parse( aLocation );
        m_cContents = cBuilder.getContentList();
    }

    /**
     * For debugging purposes: write the content list out to the output stream
     * using a simple template to format the contents.
     */
    private void debugTableOfContents() throws Exception {
        File cTocTemplate = new File( getRootPath(),
                "src/freemarker/doc/template/toc.template" );

        Template cToc = new Template( cTocTemplate );
        Writer cOut = new OutputStreamWriter( System.out );
        TemplateModelRoot cRoot = new FastHash();

        cRoot.put( "contents", m_cContents );
        cToc.process( cRoot, cOut );
        cOut.flush();
        cOut.close();
    }

    /**
     * Creates the new content files. Iterates through the populated
     * {@link ContentList}, reading a template for each entry, and outputting
     * the results into the <code>manual</code> directory.
     */
    private void createDocuments() throws IOException, TemplateModelException {
        String path = getRootPath();
        FileTemplateCache cache = new FileTemplateCache( path );

        System.out.println( "About to create documents..." );

        TemplateModelRoot modelRoot = new FastHash();
        Map utilities = new HashMap();

        utilities.put( "templateSyntaxColour", new FreeMarkerToHtml() );
        utilities.put( "javaSyntaxColour", new JavaToHtml() );
        modelRoot.put( "utility", new FastHash( utilities ));
        modelRoot.put( "contents", m_cContents );

        ContentEntry cEntry;
        Template cTemplate;
        Writer cFileOut;

        TemplateIteratorModel cIteratorModel = m_cContents.templateIterator();
        modelRoot.put( "index", cIteratorModel );

        while( cIteratorModel.hasNext() ) {
            cEntry = (ContentEntry)cIteratorModel.next();
            cTemplate = (Template)cache.getItem( "src/freemarker/doc/template/" +
                    cEntry.getLink() + ".template" );
            cFileOut = new FileWriter( new File( path, getTargetPath() +
                    cEntry.getLink() ));
            cTemplate.process( modelRoot, cFileOut );
            cFileOut.close();
        }

        System.out.println( "Finished creating documents." );
    }

    /**
     * Creates the navigation bar for the manual. This uses the same XML
     * file as the {@link #createDocuments} method, but the parsing happens
     * courtesy of the {@link freemarker.ext.jdom.NodeListModel}
     * class, and most of the output logic happens in the template.
     */
    private void makeTableOfContents() throws Exception {
        String path = getRootPath();

        SAXBuilder builder = new SAXBuilder();
        FileInputStream cInputStream;
        Document document;

        System.out.println( "Making table of contents..." );

        cInputStream = new FileInputStream( path +
                "/src/freemarker/doc/xml/contents.xml" );
        document = builder.build( cInputStream );

        TemplateModelRoot modelRoot = new FastHash();
        modelRoot.put( "document", new NodeListModel( document ));
        modelRoot.put( "compress", CompressWhitespace.getInstance() );

        Template template = new Template( path +
            "/src/freemarker/doc/template/contents.html.template" );

        Writer fileOut = new FileWriter(
                new File( path, getTargetPath() + "nav.html" ));

        template.process( modelRoot, fileOut );

        fileOut.flush();
        fileOut.close();

        System.out.println( "Finished table of contents." );

    }

    /**
     * Retrieve the root path of the FM-Classic distribution from a
     * properties file. This is used to set the root of the FM-Classic
     * cache, which in turn is used to find the example source code.
     */
    private String getRootPath() throws IOException {
        if( m_cProperties == null ) {
            loadProperties();
        }
        return (String)m_cProperties.get( "rootPath" );
    }

    /**
     * Retrieve the relative path of the FM-Classic manual from a
     * properties file. This is used to determine the destination path
     * to put the generated manual files.
     */
    private String getTargetPath() throws IOException {
        if( m_cProperties == null ) {
            loadProperties();
        }
        return (String)m_cProperties.get( "targetPath" );
    }

    /**
     * Loads properties from the <code>doc.properties</code> file.
     */
    private void loadProperties() throws IOException {
        InputStream input = getClass().getResourceAsStream( "doc.properties" );
        m_cProperties = new Properties();

        if( input != null ) {
            ((Properties)m_cProperties).load( input );
        } else {
            System.err.println( "Unable to find doc.properties file!" );
        }
        input.close();
    }

   /**
    * Main method for creating the FM-Classic manual.
    *
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        Manual cManual = new Manual();

        try {
            cManual.makeTableOfContents();
            cManual.parseContents();
            //cManual.debugTableOfContents();
            cManual.createDocuments();
        } catch( Exception e ) {
            System.err.println( "Unable to create FM-Classic manual:" );
            System.err.println( e.toString() );
        }
    }

}
