/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.ant;

import freemarker.template.*;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Enumeration;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.MatchingTask;

import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import freemarker.ext.jdom.NodeListModel;

import org.xml.sax.SAXParseException;

/**
 * <p>This is an <a href="http://ant.apache.org/" target="_top">Ant</a>
 * task for transforming XML documents using FM-Classic templates. It uses the
 * <a href="http://www.jdom.org/" target="_top">JDOM</a> adapter class, the
 * {@link freemarker.ext.jdom.NodeListModel}. It will read a set of XML
 * documents, and pass them to the template for processing, building the
 * corresponding output files in the destination directory.</p>
 *
 * <p>It makes the following variables available to the template in the data model:</p>
 * <ul>
 * <li><code>document</code>&mdash;the JDOM tree of the currently processed XML file</li>
 * <li><code>properties</code>&mdash;a {@link freemarker.template.FastHash}
 * containing properties of the project that executes the task</li>
 * <li><code>userProperties</code>&mdash;a {@link freemarker.template.FastHash}
 * containing user properties of the project that executes the task</li>
 * <li><code>project</code>&mdash;the JDOM tree of the XML file specified by the
 * <code>projectfile</code>. It will not be available if you didn't specify the
 * <code>projectfile</code> attribute. Note that this can be any XML file, not
 * necessarily the XML file of the currently executing project (altough you
 * would normally use that).</li>
 * </ul>
 *
 * <p>It supports the following attributes:</p>
 *
 * <table border="1" cellpadding="2" cellspacing="0">
 *   <tr>
 *     <th align="left">Attribute</td>
 *     <th align="left">Description</td>
 *     <th valign="top">Required</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">basedir</td>
 *     <td valign="top">location of the XML files. Defaults to the project's
 *       basedir.</td>
 *     <td align="center" valign="top">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">destdir</td>
 *     <td valign="top">location to store the generated files.</td>
 *     <td align="center" valign="top">Yes</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">includes</td>
 *     <td valign="top">comma-separated list of patterns of files that must be
 *       included; all files are included when omitted.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">includesfile</td>
 *     <td valign="top">the name of a file that contains
 *       include patterns.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">excludes</td>
 *     <td valign="top">comma-separated list of patterns of files that must be
 *       excluded; no files (except default excludes) are excluded when omitted.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">excludesfile</td>
 *     <td valign="top">the name of a file that contains exclude patterns.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">defaultexcludes</td>
 *     <td valign="top">indicates whether default excludes should be used
 *       (<code>yes</code> | <code>no</code>); default excludes are used when omitted.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">extension</td>
 *     <td valign="top">extension of generated files. Defaults to .html.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">template</td>
 *     <td valign="top">location of the FreeMarker template file that will be
 *       applied to XML files</td>
 *     <td valign="top" align="center">Yes</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">projectfile</td>
 *     <td valign="top">path to the project file. If omitted, it will not be
 *       available to templates </td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">incremental</td>
 *     <td valign="top">indicates whether all files should be regenerated (no), or
 *       only those that are older than the XML file, the template file, or the
 *       project file (yes). Defaults to yes. </td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 *   <tr>
 *     <td valign="top">encoding</td>
 *     <td valign="top">The encoding of the output files. Default to platform
 *       default encoding.</td>
 *     <td valign="top" align="center">No</td>
 *   </tr>
 * </table>
 *
 * @author Attila Szegedi, szegedia@freemail.hu
 * @author Jonathan Revusky, jon@revusky.com
 * @version $Id: FreeMarkerXmlTask.java,v 1.8 2003/11/26 05:53:48 run2000 Exp $
 */
public class FreeMarkerXmlTask extends MatchingTask
{
	/** The <code>SAXBuilder</code> instance to use */
	private SAXBuilder builder;

	/** the destination directory */
	private File destDir = null;

	/** the base directory */
	private File baseDir = null;

	/** the template attribute */
	private String template = null;

	/** The template in its parsed form */
	private Template parsedTemplate;

	/** last modified of the template sheet */
	private long templateFileLastModified = 0;

	/** the projectFile attribute */
	private String projectAttribute = null;

    /** The projectFile attribute as a <code>File</code> object */
	private File projectFile = null;

	/** The JDOM tree of the project wrapped into FreeMarker TemplateModel */
	private TemplateModel projectTemplate;
    /** The properties of the Ant project as a FreeMarker TemplateModel */
	private TemplateModel propertiesTemplate;
    /** The user properties of the Ant project as a FreeMarker TemplateModel */
	private TemplateModel userPropertiesTemplate;

	/** last modified of the project file if it exists */
	private long projectFileLastModified = 0;

	/** check the last modified date on files, defaults to true */
	private boolean incremental = true;

	/** the default output extension is .html */
	private String extension = ".html";

    /** The output encoding of the applied templates */
	private String encoding = null;

	/**
	 * Constructor creates the SAXBuilder.
	 */
	public FreeMarkerXmlTask()
	{
		builder = new SAXBuilder();
	}

	/**
	 * Set the base directory. Defaults to <code>.</code>
     *
     * @param dir the name of the base input directory
	 */
	public void setBasedir(File dir)
	{
		baseDir = dir;
	}

	/**
	 * Set the destination directory into which the generated
	 * files should be copied to
     *
	 * @param dir the name of the destination directory
	 */
	public void setDestdir(File dir)
	{
		destDir = dir;
	}

	/**
	 * Set the output file extension. <code>.html</code> by default.
     *
     * @param extension the file extension for the output files
	 */
	public void setExtension(String extension)
	{
		this.extension = extension;
	}

	/**
	 * Set the path to the template file
     *
     * @param template the template to be used when transforming the XML
	 */
	public void setTemplate(String template)
	{
		this.template = template;
	}

	/**
	 * Set the path to the project XML file
     *
     * @param projectAttribute the project attribute from the Ant task
	 */
	public void setProjectfile(String projectAttribute)
	{
		this.projectAttribute = projectAttribute;
	}

	/**
	 * Turn on/off incremental processing. On by default
	 */
	public void setIncremental(String incremental)
	{
		this.incremental = !(incremental.equalsIgnoreCase("false") || incremental.equalsIgnoreCase("no") || incremental.equalsIgnoreCase("off"));
	}

	/**
	 * Set encoding for generated files. Defaults to platform default encoding.
     *
     * @param encoding the encoding to be used
	 */
	public void setEncoding(String encoding)
	{
		this.encoding = encoding;
	}

    /**
     * Execute this task as an Ant task.
     */
	public void execute() throws BuildException
	{
		DirectoryScanner scanner;
		String[]         list;

		if(baseDir == null)
		{
			baseDir = project.resolveFile(".");
		}
		if(destDir == null )
		{
			String msg = "destdir attribute must be set!";
			throw new BuildException(msg, getLocation());
		}
		if (template == null)
		{
			throw new BuildException("template attribute must be set!", getLocation());
		}
		File templateFile = new File(template);
                if (!templateFile.isFile() && !templateFile.isAbsolute()) {
                    templateFile = new File(getProject().getBaseDir(), template);
                }
		if(!templateFile.isFile())
			throw new BuildException("template file cannot be found:" + templateFile.getAbsolutePath(), getLocation());

        FileTemplateCache templateCache = new FileTemplateCache(templateFile.getAbsoluteFile().getParentFile());
        parsedTemplate = (Template)templateCache.getItem(templateFile.getName());
        parsedTemplate.setCache(templateCache);

        // get the last modification of the template
		templateFileLastModified = templateFile.lastModified();

		log("Transforming into: " + destDir.getAbsolutePath(), Project.MSG_INFO);

		// projectFile relative to baseDir
		if (projectAttribute != null && projectAttribute.length() > 0)
		{
			projectFile = new File(baseDir, projectAttribute);
			if (projectFile.isFile())
				projectFileLastModified = projectFile.lastModified();
			else
			{
				log ("Project file is defined, but could not be located: " +
						 projectFile.getAbsolutePath(), Project.MSG_INFO );
				projectFile = null;
			}
		}

		// find the files/directories
		scanner = getDirectoryScanner(baseDir);

		propertiesTemplate = hashtableToTemplate(project.getProperties());
		userPropertiesTemplate = hashtableToTemplate(project.getUserProperties());

		// get a list of files to work on
		list = scanner.getIncludedFiles();
		for (int i = 0;i < list.length; ++i)
		{
			process(baseDir, list[i], destDir);
		}
	}

	/**
	 * Process an XML file using FreeMarker
     *
     * @param baseDir the directory that the given <code>xmlFile</code> is
     * relative to
     * @param xmlFile the XML file to be used as input for the template
     * processing
     * @param destDir the directory where the transformed XML file will be
     * output
	 */
	private void process(File baseDir, String xmlFile, File destDir)
		throws BuildException
	{
		File   outFile=null;
		File   inFile=null;
		try
		{
			// the current input file relative to the baseDir
			inFile = new File(baseDir,xmlFile);
			// the output file relative to basedir
			outFile = new File(destDir,
				xmlFile.substring(0,
				xmlFile.lastIndexOf('.')) + extension);

			// only process files that have changed
			if (incremental == false ||
				(inFile.lastModified() > outFile.lastModified() ||
				templateFileLastModified > outFile.lastModified() ||
				projectFileLastModified > outFile.lastModified()))
			{
				ensureDirectoryFor( outFile );

				//-- command line status
				log("Input:  " + xmlFile, Project.MSG_INFO );

				// Build the file JDOM
				TemplateModel document = new NodeListModel(builder.build(inFile));
				// Build the project JDOM on-demand
				if (projectTemplate == null && projectFile != null)
					projectTemplate = new NodeListModel(builder.build(projectFile));

				TemplateModelRoot root = new FastHash();
				root.put("document", document);
				root.put("properties", propertiesTemplate);
				root.put("userProperties", userPropertiesTemplate);
				if(projectTemplate != null)
					root.put("project", projectTemplate);

				// Process the template and write out
				// the result as the outFile.
				Writer writer = (encoding == null ? new FileWriter(outFile) : new OutputStreamWriter(new FileOutputStream(outFile), encoding));
				try
				{
					parsedTemplate.process(root, writer);
					writer.flush();
				}
				finally
				{
					writer.close();
				}

				log("Output: " + outFile, Project.MSG_INFO );
			}
		}
		catch (JDOMException e)
		{
			if (outFile != null ) outFile.delete();
			Throwable rootCause = e;
			if (e.getCause() != null)
			{
				rootCause = e.getCause();
				if (rootCause instanceof SAXParseException)
				{
					SAXParseException spe = (SAXParseException)rootCause;
					if(spe.getException() != null)
						rootCause = spe.getException();
					log("XML parsing error in " + inFile.getAbsolutePath(), Project.MSG_ERR);
					log("Line number " + spe.getLineNumber());
					log("Column number " + spe.getColumnNumber());
				}
			}
			throw new BuildException(rootCause, getLocation());
		}
		catch (Throwable e)
		{
			if (outFile != null ) outFile.delete();
                        e.printStackTrace();
			throw new BuildException(e, getLocation());
		}
	}

	/**
	 * Create directories as needed
     *
     * @param targetFile the target whose directory may need to be created.
	 */
	private void ensureDirectoryFor( File targetFile ) throws BuildException
	{
		File directory = new File( targetFile.getParent() );
		if (!directory.exists())
		{
			if (!directory.mkdirs())
			{
				throw new BuildException("Unable to create directory: "
					+ directory.getAbsolutePath(), getLocation());
			}
		}
	}

    /**
     * Create a {@link freemarker.template.TemplateModel} based on key-value
     * pairs in the given <code>Hashtable</code>. The elements of the table
     * are copied rather than using an adapter class for the <code>Hashtable</code>
     * itself.
     *
     * @param table the <code>Hashtable</code> containing the values to be
     * copied
     * @return a {@link freemarker.template.TemplateModelRoot} containing all
     * the key-value pairs in the <code>Hashtable</code>
     */
	private static TemplateModel hashtableToTemplate(Hashtable table)
	{
		TemplateModelRoot model = new FastHash();
		Enumeration keys = table.keys();
		while(keys.hasMoreElements())
		{
			Object key = keys.nextElement();
			Object value = table.get(key);
			model.put(key.toString(), new SimpleScalar(value.toString()));
		}
		return model;
	}
}
