/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.beans;

import freemarker.template.*;
import java.util.*;

/**
 * <p>A class that adds {@link TemplateListModel2} functionality to the
 * {@link Enumeration} interface implementers.</p>
 *
 * <p>Using the model as a list model is <em>NOT</em> thread-safe, as
 * enumerations are inherently not thread-safe.</p>
 *
 * @author Attila Szegedi, attila@szegedi.org
 * @version $Id: EnumerationModel.java,v 1.5 2003/11/26 05:53:48 run2000 Exp $
 */
public final class EnumerationModel extends ObjectModel implements TemplateListModel2
{
	/**
	 * Creates a new model that wraps the specified enumeration object.
	 * @param enumeration the enumeration object to wrap into a model.
	 */
	public EnumerationModel(Enumeration enumeration)
	{
		super(enumeration);
	}

    /**
     * Returns true if the enumeration is null or the enumeration has no
     * more elements, otherwise false.
     */
    public boolean isEmpty()
    {
        return object == null || !((Enumeration)object).hasMoreElements();
    }

	/**
	 * Returns a model wrapping the specified enumeration object. It always
     * returns a new object, as Enumerations are considered short-lived and
     * caching them would make little sense.
     *
	 * @return the model for the enumeration
	 */
	public static final EnumerationModel getInstance(Enumeration object)
	{
		return new EnumerationModel(object);
	}

	/**
	 * Returns the type of this object (which is TYPE_ENUMERATION).
	 */
	public int getType()
	{
		return TYPE_ENUMERATION;
	}

    /**
     * Retrieve a {@link TemplateIteratorModel} for this enumeration.
     *
     * @return the iterator model for walking over this enumeration
     * @throws TemplateModelException the enumeration has already been spent
     */
    public TemplateIteratorModel templateIterator() throws TemplateModelException {
        if( object == null ) {
            throw new TemplateModelException( "Enumerator is already finished" );
        }
        return new EnumerationModelImpl();
    }

    /**
     * Release the underlying enumerator, so that we don't try to walk over
     * it again
     *
     * @param iterator the iterator that just walked over the enumeration
     */
    public void releaseIterator( TemplateIteratorModel iterator ) {
        object = null;
    }

    /**
     * An enumeration wrapped in a {@link TemplateIteratorModel}. This provides
     * the iterator implementation for the parent class.
     */
    private class EnumerationModelImpl implements TemplateIteratorModel {
        /**
         * Calls {@link Enumeration#hasMoreElements()}.
         */
        public boolean hasNext()
        {
            return ((Enumeration)object).hasMoreElements();
        }

        /**
         * Calls {@link Enumeration#nextElement()}.
         */
        public TemplateModel next()
        {
            return BeansWrapper.wrap(((Enumeration)object).nextElement());
        }

        /**
         * Returns true if the enumeration is null or the enumeration has no
         * more elements, otherwise false.
         */
        public boolean isEmpty()
        {
            return object == null || !((Enumeration)object).hasMoreElements();
        }
    }
}
