/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.beans;

import freemarker.template.*;
import java.util.*;

/**
 * <p>A special case of {@link ObjectModel} that adds implementation
 * for {@link freemarker.template.TemplateMethodModel2} on map objects that is a
 * shortcut for the <code>Map.get()</code> method.</p>
 *
 * <p>You can use <code>map.entrySet</code>, <code>map.keySet</code>, or
 * <code>map.values</code> to obtain {@link TemplateListModel2} instances
 * for various aspects of the map.</p>
 *
 * @author Attila Szegedi, attila@szegedi.org
 * @version $Id: MapModel.java,v 1.4 2003/10/20 13:49:02 run2000 Exp $
 */
public class MapModel
extends
	ObjectModel
implements
	TemplateMethodModel2
{

	/**
	 * Creates a new model that wraps the specified map object.
	 * @param map the map object to wrap into a model.
	 */
	public MapModel(Map map)
	{
		super(map);
	}

	/**
	 * Returns a model wrapping the specified map object. If there is already
	 * a cached model instance for this map, returns the cached model instance.
	 * Models are cached using <code>WeakReference</code> objects. The caching can be turned
	 * off by setting the <kbd>expose.reflection.nocache</kbd> system property to
	 * true. In this case calling this method is equivalent to constructing a new model.
	 * @param object the map to wrap into a model.
	 * @return the model for the map
	 */
	public static final MapModel getInstance(Map object)
	{
		if(noCache)
			return new MapModel(object);

		MapModel model = (MapModel)lookup(object);
		if(model == null)
		{
			model = new MapModel(object);
			register(model, object);
		}
		return model;
	}

	/**
	 * Returns the type of this object (which is TYPE_MAP)
	 */
	public int getType()
	{
		return TYPE_MAP;
	}

	/**
	 * The first argument is used as a key to call the map's
     * <code>get</code> method.
	 */
	public TemplateModel exec(List arguments) throws TemplateModelException
	{
        if( arguments.size() != 1 ) {
            throw new TemplateModelException( "MapModel must have exactly one argument" );
        }
		return BeansWrapper.wrap(((Map)object).get(BeansWrapper.unwrap(arguments.get(0))));
	}

	/**
	 * Returns true if the underlying map is empty.
	 */
	public boolean isEmpty()
	{
		return ((Map)object).isEmpty();
	}
}
