/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.jsp;

import javax.servlet.jsp.tagext.*;

import java.io.*;

import javax.servlet.ServletException;
import javax.servlet.jsp.tagext.BodyTag;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.BodyContent;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import freemarker.template.*;

/**
 * Simple implementation of JSP tag to allow use of FM-Classic templates in
 * JSP. Inspired by similar class in Velocity template engine developed by
 * <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
 *
 * @author Attila Szegedi, szegedia@freemail.hu
 */
public class FreeMarkerTag implements BodyTag
{
    private Tag         parent = null;
    private BodyContent bodyContent = null;
    private PageContext pageContext = null;
    private Template    template = null;
    private boolean     caching = true;

    /**
     * Retrieve whether we're caching template content from the body, or
     * whether we recompile the template each time.
     *
     * @return <code>true</code> if we're caching the template, otherwise
     * <code>false</code>
     */
    public boolean getCaching()
    {
        return caching;
    }

    /**
     * Sets whether we cache the template content from the body, or whether
     * we recompile the template each time.
     *
     * @param caching <code>true</code> if we want to cache template content,
     * otherwise <code>false</code>
     */
    public void setCaching(boolean caching)
    {
        this.caching = caching;
    }

    /**
     * Get the parent (closest enclosing tag handler) for this tag handler.
     */
    public Tag getParent()
    {
        return parent;
    }

    /**
     * <p>Set the parent (closest enclosing tag handler) of this tag handler.
     * Invoked by the JSP page implementation object prior to doStartTag().</p>
     *
     * <p>This value is <em>not</em> reset by {@link #doEndTag} and must be
     * explicitly reset by a page implementation.</p>
     *
     * @param parent The parent tag, or <code>null</code>.
     */
    public void setParent(Tag parent)
    {
        this.parent = parent;
    }

    /**
     * Process the start tag for this instance. This method is invoked by the
     * JSP page implementation object.
     */
    public int doStartTag()
    {
        return EVAL_BODY_TAG;
    }

    /**
     * Set the bodyContent property.
     */
    public void setBodyContent(BodyContent bodyContent)
    {
        this.bodyContent = bodyContent;
    }

    /**
     * <p>Set the current page context. This method is invoked by the JSP page
     * implementation object prior to {@link #doStartTag}.</p>
     *
     * <p>This value is <em>not</em> reset by {@link #doEndTag} and must be
     * explicitly reset by a page implementation if it changes between calls to
     * {@link #doStartTag}.</p>
     */
    public void setPageContext(PageContext pageContext)
    {
        this.pageContext = pageContext;
    }

    /**
     * Prepare for evaluation of the body.
     */
    public void doInitBody()
    {
    }

    /**
     * Process body (re)evaluation. This method is invoked by the JSP Page
     * implementation object after every evaluation of the body into the
     * BodyEvaluation object. The method is not invoked if there is no
     * body evaluation.
     *
     * @return whether additional evaluations of the body are desired
     */
    public int doAfterBody()
    {
        return 0;
    }

    /**
     * Called on a Tag handler to release any state information.
     */
    public void release()
    {
    }

    /**
     * <p>Process the end tag for this instance. This method is invoked by the
     * JSP page implementation object on all Tag handlers.</p>
     *
     * <p>This method will be called after returning from {@link #doStartTag}.
     * The body of the action may or not have been evaluated, depending on the
     * return value of {@link #doStartTag}.</p>
     */
    public int doEndTag() throws JspException
    {
        if (bodyContent == null)
            return EVAL_PAGE;

        try
        {
            TemplateModelRoot root = new FastHash();
            root.put("page", new JspContextModel(pageContext, JspContextModel.PAGE_SCOPE));
            root.put("request", new JspContextModel(pageContext, JspContextModel.REQUEST_SCOPE));
            root.put("session", new JspContextModel(pageContext, JspContextModel.SESSION_SCOPE));
            root.put("application", new JspContextModel(pageContext, JspContextModel.APPLICATION_SCOPE));
            root.put("any", new JspContextModel(pageContext, JspContextModel.ANY_SCOPE));

            if(template == null || !caching)
                template = new Template(bodyContent.getReader());

            template.process(root, pageContext.getOut());
        }
        catch(Exception e)
        {
            try
            {
                pageContext.handlePageException(e);
            }
            catch(ServletException e2)
            {
                throw new JspException(e2.getMessage());
            }
            catch(IOException e2)
            {
                throw new JspException(e2.getMessage());
            }
        }

        return EVAL_PAGE;
    }
}
