/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
package freemarker.ext.misc;

import java.util.List;

import freemarker.template.*;

/**
 * <p>Performs a type conversion from a number model to a String model. This is
 * useful when we need to perform String concatenation on two numbers,
 * or when we only want a number from a multimodel.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "asString", freemarker.ext.misc.AsString.getInstance() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * &lt;assign var1 = 5&gt;
 * &lt;assign var2 = 3&gt;
 * &lt;assign var3 = asString( var1 ) + asString( var2 )&gt;
 * &lt;p&gt;The concatenation of ${var1} and ${var2} is: ${var3}&lt;/p&gt;
 *
 * ...
 * </pre>
 *
 * <p><b>Note:</b><br />
 * <code>AsString</code> is a singleton instance. Use the
 * {@link #getInstance()} method to retrieve instances of this model.</p>
 *
 * @author Nicholas Cull
 * @version $Id: AsString.java,v 1.5 2003/11/17 13:06:24 run2000 Exp $
 * @since 1.8
 */
public final class AsString implements TemplateMethodModel2 {
    private static final AsString instance = new AsString();

    /**
     * Private constructor, meaning this shouldn't be instantiated directly.
     * Use the {@link #getInstance} method to return an instance of this
     * class.
     */
    private AsString() {
    }

    /**
     * Retrieve a singleton instance of this class. Since there is no
     * state information held between calls, there is only a need for
     * one instance.
     *
     * @return an instance of this <code>AsString</code> class
     */
    public static AsString getInstance() {
        return instance;
    }

    /**
     * Perform a number to string conversion on the given template model.
     *
     * @param arguments a List of exactly one template model to be converted
     * @return a new <code>TemplateScalarModel</code> of the String value
     * @throws TemplateModelException More than one argument is supplied,
     * or the model is neither a number nor a string model.
     */
    public TemplateModel exec( List arguments ) throws TemplateModelException {
        if( arguments.size() != 1 ) {
            throw new TemplateModelException( "ToString must have exactly one argument" );
        }
        TemplateModel model = (TemplateModel)arguments.get(0);
        if( model instanceof TemplateScalarModel ) {
            return new FastScalar( ((TemplateScalarModel)model).getAsString() );
        }
        if( model instanceof TemplateNumberModel ) {
            String stringValue = Long.toString( ((TemplateNumberModel)model).getAsNumber() );
            return new FastScalar( stringValue );
        }

        throw new TemplateModelException( "ToString must be given a number or string argument" );
    }

    /**
     * Is this model empty?
     *
     * @return <code>false</code>, to indicate the model is non-empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }
}
