/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.ext.misc;
import freemarker.template.*;
import java.util.List;

/**
 * <p>Allows numeric values in a data model to be decremented. The result is
 * always an integer value.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "decrement", new freemarker.ext.misc.Decrement() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * &lt;assign var1 = 5&gt;
 * &lt;assign var1 = decrement( var1 )&gt;
 * &lt;p&gt;The decremented value of var1 is: ${var1}&lt;/p&gt;
 *
 * ...
 * </pre>
 *
 * @version $Id: Decrement.java,v 1.8 2003/11/17 13:06:24 run2000 Exp $
 * @deprecated With the introduction of native numbers and numeric operators
 * in 1.8, this implementation is now deprecated
 */
public class Decrement implements TemplateMethodModel {

    /** Creates new Decrement */
    public Decrement() {
    }

    /**
     * Executes a method call.
     *
     * @param arguments a <code>List</code> of <code>String</code> objects
     * containing the values of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec(List arguments) throws TemplateModelException {
        int nNumber;

        if( arguments.size() != 1 ) {
            throw new TemplateModelException(
                    "Decrement operator must have exactly one argument" );
        }

        try {
            nNumber = Integer.parseInt((String)arguments.get(0));
        } catch( NumberFormatException e ) {
            throw new TemplateModelException( "Could not parse argument as a number", e );
        }

        return new FastScalar( Integer.toString( nNumber - 1 ));
    }

    /**
     * Is the object empty?
     *
     * @return <code>false</code>, to indicate the object is not empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }
}
