/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.ext.misc;

import freemarker.template.*;
import java.io.*;
import java.util.*;

/**
 * <p>Gives FM-Classic the the ability to execute external commands. Will fork
 * a process, and inline anything that process sends to stdout in the template.
 * Based on a patch submitted by Peter Molettiere.</p>
 *
 * <p>BE CAREFUL! this tag, depending on use, may allow you
 * to set something up so that users of your web
 * application could run arbitrary code on your server.
 * This can only happen if you allow unchecked GET/POST
 * submissions to be used as the command string in the
 * exec method.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "exec", freemarker.ext.misc.Execute.getInstance() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * The following is executed:
 * ${exec( "/usr/bin/ls" )}
 *
 * ...
 * </pre>
 *
 * <p><b>Note:</b><br />
 * <code>Execute</code> is a singleton instance. Use the
 * {@link #getInstance()} method to retrieve instances of this model.</p>
 *
 * @version $Id: Execute.java,v 1.8 2003/11/17 13:06:24 run2000 Exp $
 */
public final class Execute implements freemarker.template.TemplateMethodModel {
    private final static int OUTPUT_BUFFER_SIZE = 1024;
    private static final Execute instance = new Execute();

    /**
     * Creates new Execute template method model.
     *
     * @deprecated use the {@link #getInstance} method to avoid excessive
     * object creation
     */
    public Execute () {
    }

    /**
     * Retrieve a singleton instance of this class. Since there is no
     * state information held between calls, there is only a need for
     * one instance.
     *
     * @return an instance of this <code>Execute</code> class
     */
    public static Execute getInstance() {
        return instance;
    }

    /**
     * Executes a method call.
     *
     * @param arguments a <code>List</code> of <code>String</code> objects containing the values
     * of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec (List arguments) throws TemplateModelException {
        String aExecute;
        StringBuffer    aOutputBuffer = new StringBuffer();
        TemplateModel   cModel;

        if( arguments.size() < 1 ) {
            throw new TemplateModelException( "Need an argument to execute" );
        }

        aExecute = (String)(arguments.get(0));

        try {
            Process exec = Runtime.getRuntime().exec( aExecute );

            // stdout from the process comes in here
            InputStream execOut = exec.getInputStream();
            Reader execReader = new InputStreamReader( execOut );

            char[] buffer = new char[ OUTPUT_BUFFER_SIZE ];
            int bytes_read = execReader.read( buffer );

            while( bytes_read > 0 ) {
                aOutputBuffer.append( buffer, 0, bytes_read );
                bytes_read = execReader.read( buffer );
            }
        }
        catch( IOException ioe ) {
            throw new TemplateModelException( ioe.getMessage() );
        }
        finally {
            cModel = new FastScalar( aOutputBuffer.toString() );
            return cModel;
        }
    }

    /**
     * Is the object empty?
     *
     * @return <code>false</code>, to indicate the object is not empty
     */
    public boolean isEmpty () throws TemplateModelException {
        return false;
    }
}
