/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.ext.misc;
import freemarker.template.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.io.Serializable;

/**
 * <p>Simple extension to the {@link freemarker.template.SimpleHash} implementation.
 * This class implements a number of convenience facilities in the form of template
 * hash keys. The hash keys are:</p>
 * <ul>
 * <li><code>_size</code> - retrieves the number of elements in the hash</li>
 * <li><code>_keys</code> - returns an <code>ExtendedList</code> model containing all
 * keys in the hash</li>
 * <li><code>_values</code> - returns an <code>ExtendedList</code> model containing all
 * values in the hash</li>
 * </ul>
 *
 * @version $Id: ExtendedHash.java,v 1.7 2003/11/16 05:00:07 run2000 Exp $
 * @since 1.7
 * @see ExtendedList
 */
public class ExtendedHash extends SimpleHash implements Serializable {

    /** Class UUID for serialization. */
    private static final long serialVersionUID = 8661840173354127769L;

    /** Creates new <code>ExtendedHash</code>. */
    public ExtendedHash() {
        super();
    }

    /** Creates new <code>ExtendedHash</code> given a backing <code>Map</code>. */
    public ExtendedHash(Map hash) {
        super( hash );
    }

    /**
     * Gets a {@link freemarker.template.TemplateModel} from the hash. If the
     * key matches one of the special values used in this implementation,
     * return the special value, otherwise return the value from the underlying
     * <code>Map</code>.
     *
     * @param key the name by which the <code>TemplateModel</code>
     * is identified in the template.
     * @return the <code>TemplateModel</code> referred to by the key,
     * or <code>null</code> if not found.
     */
    public synchronized TemplateModel get(String key)
            throws TemplateModelException {

        if(( key == null ) || ( key.length() == 0 ) || ( key.charAt( 0 ) != '_' )) {
            return (TemplateModel)hash.get(key);
        }
        if( key.equals( "_keys" )) {
            return new ExtendedList( toScalarList( hash.keySet() ));
        }
        if( key.equals( "_values" )) {
            return new ExtendedList( new ArrayList( hash.values() ));
        }
        if( key.equals( "_size" )) {
            return new FastNumber( hash.size() );
        }
        return (TemplateModel)hash.get(key);
    }

    /**
     * Turns a collection of <code>String</code>s into a <code>List</code> of
     * {@link freemarker.template.SimpleScalar} objects. This is required
     * in order to get suitable values from a keyset of the underlying hash.
     *
     * @param list a <code>Collection</code> of String values to be translated
     * into <code>SimpleScalar</code>s.
     * @return a <code>List</code> of <code>SimpleScalar</code>s
     */
    private List toScalarList( Collection list ) {
        ArrayList newList = new ArrayList( list.size() );
        Iterator iterator = list.iterator();

        while( iterator.hasNext() ) {
            newList.add( new FastScalar( (String) iterator.next() ));
        }

        return newList;
    }
}
