/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.ext.misc;
import freemarker.template.*;
import java.util.List;

/**
 * <p>Compares two scalar data models as numbers. Integer comparison is
 * performed.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "greaterThanOrEquals", new freemarker.ext.misc.GreaterThanOrEquals() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * &lt;assign temp = 2&gt;
 * &lt;if greaterThanOrEquals( temp, 4 )&gt;
 *   &lt;p&gt;Two is greater than or equal to four!&lt;/p&gt;
 * &lt;/if&gt;
 *
 * ...
 * </pre>
 *
 * @version $Id: GreaterThanOrEquals.java,v 1.8 2003/11/17 13:06:24 run2000 Exp $
 * @deprecated With the introduction of native numbers and numeric operators
 * in 1.8, this implementation is now deprecated
 */
public class GreaterThanOrEquals implements TemplateMethodModel {

    /** Creates new GreaterThanOrEquals */
    public GreaterThanOrEquals() {
    }

    /**
     * Executes a method call.
     *
     * @param arguments a <code>List</code> of <code>String</code> objects
     * containing the values of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec(List arguments) throws TemplateModelException {
        int nNumber1, nNumber2;

        if( arguments.size() != 2 ) {
            throw new TemplateModelException(
                "LessThan operator needs exactly two arguments."
                );
        }

        try {
            nNumber1 = Integer.parseInt((String)arguments.get(0));
            nNumber2 = Integer.parseInt((String)arguments.get(1));

        } catch(NumberFormatException e) {
            throw new TemplateModelException(
                "One or more numbers could not be parsed as an integer."
                );
        }

        return FastBoolean.getInstance( nNumber1 >= nNumber2 );
    }

    /**
     * Is the object empty?
     *
     * @return <code>false</code>, to indicate this object is not empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }

}
