/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
package freemarker.ext.misc;

import java.util.*;

import freemarker.template.*;

/**
 * <p>Determines the keyset of the underlying <code>Map</code> object given a
 * <code>TemplateObjectModel</code>. Does this by calling the
 * <code>getAsObject</code> method. If the object is of the expected type,
 * the keyset is returned as a <code>FastList</code>. Otherwise, a
 * <code>TemplateModelException</code> is thrown.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "keys", freemarker.ext.misc.HashKeys.getInstance() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * &lt;assign hash1 = { "test1", "value1",
 *                      "test2", "value2",
 *                      "test3", "value3" }&gt;
 * &lt;assign keyList = keys( hash1 )&gt;
 * &lt;p&gt;The keys of hash1 are:&lt;/p&gt;
 * &lt;p&gt;&lt;foreach key in keyList&gt;
 * ${key}&lt;br&gt;
 * &lt;/foreach&gt;&lt;/p&gt;
 *
 * ...
 * </pre>
 *
 * <p><b>Note:</b><br />
 * <code>HashKeys</code> is a singleton instance. Use the
 * {@link #getInstance()} method to retrieve instances of this model.</p>
 *
 * @author Nicholas Cull
 * @version $Id: HashKeys.java,v 1.1 2003/11/17 13:05:37 run2000 Exp $
 * @since 1.8
 */
public final class HashKeys implements TemplateMethodModel2 {

    private static final HashKeys instance = new HashKeys();

    /** Private constructor, to indicate this is a singleton class. */
    private HashKeys() {
    }

    /**
     * Retrieve a singleton instance of this class. Since there is no
     * state information held between calls, there is only a need for
     * one instance.
     *
     * @return an instance of this <code>HashKeys</code> class
     */
    public static HashKeys getInstance() {
        return instance;
    }

    /**
     * Is the object empty?
     *
     * @return <code>false</code>, since the method is never empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }

    /**
     * Executes a method call. Arguments are passed as a <code>List</code> of
     * {@link TemplateModel} objects.
     *
     * @param arguments a <code>List</code> of <code>TemplateModel</code> objects
     * containing the values of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec( List arguments ) throws TemplateModelException {
        if( arguments.size() != 1 ) {
            throw new TemplateModelException( "HashKeys method must have exactly one argument" );
        }

        if(!( arguments.get(0) instanceof TemplateObjectModel )) {
            throw new TemplateModelException( "Argument must implement the TemplateObjectModel interface" );
        }
        TemplateObjectModel model = (TemplateObjectModel)arguments.get(0);
        Object object = model.getAsObject();

        if( !( object instanceof Map )) {
            throw new TemplateModelException( "Could not determined keys of this object" );
        }

        Set keySet = ((Map)object).keySet();
        ArrayList list = new ArrayList(keySet.size());
        Iterator iKey = keySet.iterator();
        while(iKey.hasNext()) {
            list.add( new FastScalar( (String)iKey.next() ));
        }
        return new FastList( list );
    }
}
