/*
 * HtmlFormattedExceptionListener.java
 *
 * Created on 2 December 2001, 17:44
 */

package freemarker.ext.misc;

import freemarker.template.TemplateExceptionListener;
import freemarker.template.TemplateExceptionEvent;
import freemarker.template.compiler.ParseException;

import java.io.Writer;
import java.io.IOException;

/**
 * <p>Exception listener that improves reporting for compile errors.
 * Modified error reporting so that errors detected at compile time
 * are returned as an entire HTML page rather than a comment.
 * The error text is displayed on the page (<em>not</em> inside of
 * a comment).</p>
 *
 * @author  <a href="run2000@users.sourceforge.net">Nicholas Cull</a>, based
 * on code by Dan Milstein
 * @version $Id: HtmlFormattedExceptionListener.java,v 1.4 2003/11/17 13:06:24 run2000 Exp $
 */
public class HtmlFormattedExceptionListener implements TemplateExceptionListener {

    private static final String COMMENT_START = "<!-- ";
    private static final String COMMENT_END = " -->";
    private static final String ERROR_PAGE_START = "<html><head></head><body bgcolor=\"#ffffff\">";
    private static final String ERROR_PAGE_END = "</body></html>";

    /** Creates a new instance of HtmlFormattedExceptionListener */
    public HtmlFormattedExceptionListener() {
    }

    /**
     * Handles events that are created whenever an exception is thrown.
     * This handler turns the event into either an HTML comment, or a
     * fully-formed HTML page, depending on whether its a compile-time or
     * run-time error. The result is inserted into the current output stream
     * of the Template.
     */
    public void exceptionThrown(TemplateExceptionEvent e) {
        Exception ex = e.getException();
        Writer out = e.getWriter();
        String message = ex.getMessage();
        String newline = System.getProperty( "line.separator" );

        try {
            if( ex instanceof ParseException ) {
                out.write( ERROR_PAGE_START );
                out.write( newline );
                out.write( "Template Compilation Error: " );
                out.write( message );
                out.write( newline );
                out.write( ERROR_PAGE_END );
                out.write( newline );
            } else {
                out.write( COMMENT_START );
                out.write( "Template Error: " );
                out.write( message );
                out.write( COMMENT_END );
                out.write( newline );
            }
        } catch( IOException e1 ) {
            // Nothing we can do here.
            // PrintWriter behaviour was to ignore IOExceptions.
        }
    }

}
