/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
package freemarker.ext.misc;

import java.util.List;

import freemarker.template.*;

/**
 * <p>Determines whether a given model is an instance of a
 * <code>TemplateListModel</code> or <code>TemplateIndexedModel</code>.</p>
 *
 * <p><b>Note:</b><br />
 * <code>IsIndexed</code> is a singleton instance. Use the
 * {@link #getInstance()} method to retrieve instances of this model.</p>
 *
 * @author Nicholas Cull
 * @version $Id: IsIndexed.java,v 1.2 2003/11/17 13:06:24 run2000 Exp $
 * @since 1.8
 */
public final class IsIndexed implements TemplateMethodModel2 {

    private static final IsIndexed instance = new IsIndexed();

    /** Private constructor, indicating that this is a singleton. */
    private IsIndexed() {
    }

    /**
     * Return a singleton instance of this class.
     *
     * @return a new IsIndexed object
     */
    public static IsIndexed getInstance() {
        return instance;
    }

    /**
     * Is the object empty?
     *
     * @return <code>true</code> if this object is empty, otherwise <code>false</code>
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }

    /**
     * Executes a method call. Arguments are passed as a <code>List</code> of
     * {@link TemplateModel} objects.
     *
     * @param arguments a <code>List</code> of <code>TemplateModel</code> objects
     * containing the values of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec( List arguments ) throws TemplateModelException {
        if( arguments.size() != 1 ) {
            throw new TemplateModelException(
                    "IsIndexed method needs exactly one argument" );
        }
        TemplateModel model = (TemplateModel)arguments.get(0);
        boolean match = ( model instanceof TemplateIndexedModel ) ||
                        ( model instanceof TemplateListModel );
        return FastBoolean.getInstance( match );
    }
}
