/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.ext.misc;
import freemarker.template.*;
import java.util.List;

/**
 * <p>Allows two numeric template data models to be subtracted. Integer
 * subtraction is performed.</p>
 *
 * <p>Usage:<br />
 * From java:</p>
 * <pre>
 * TemplateModelRoot root = new SimpleHash();
 *
 * root.put( "subtract", new freemarker.ext.misc.Subtract() );
 *
 * ...
 * </pre>
 *
 * <p>From your FM-Classic template:</p>
 * <pre>
 * &lt;assign var1 = 5&gt;
 * &lt;assign var2 = 3&gt;
 * &lt;assign var3 = subtract( var1, var2 )&gt;
 * &lt;p&gt;The difference of ${var1} and ${var2} is: ${var3}&lt;/p&gt;
 *
 * ...
 * </pre>
 *
 * @version $Id: Subtract.java,v 1.8 2003/11/17 13:06:24 run2000 Exp $
 * @since 1.7
 * @deprecated With the introduction of native numbers and numeric operators
 * in 1.8, this implementation is now deprecated
 */
public class Subtract implements TemplateMethodModel {

    /** Creates new Subtract */
    public Subtract() {
    }

    /**
     * Executes a method call.
     *
     * @param arguments a <code>List</code> of <code>String</code> objects
     * containing the values of the arguments passed to the method.
     * @return the <code>TemplateModel</code> produced by the method, or <code>null</code>.
     */
    public TemplateModel exec(List arguments) throws TemplateModelException {
        int nNumber1, nNumber2;

        if (arguments.size() != 2) {
            throw new TemplateModelException(
                    "Subtract operation must involve exactly two arguments" );
        }

        try {
            nNumber1 = Integer.parseInt((String)arguments.get(0));
            nNumber2 = Integer.parseInt((String)arguments.get(1));

        } catch(NumberFormatException e) {
            throw new TemplateModelException(
                "One or more numbers could not be parsed as an integer", e
                );
        }

        return new FastScalar( Integer.toString( nNumber1 - nNumber2 ));
    }

    /**
     * Is the object empty?
     *
     * @return <code>false</code>, to indicate the object is not empty
     */
    public boolean isEmpty() throws TemplateModelException {
        return false;
    }

}
