/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.servlet;

import freemarker.template.*;

import java.io.*;
import java.util.*;
import java.text.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * This is a general-purpose FreeMarker servlet with following features:
 * <ul>
 * <li>It utilizes a TemplateCache that is fully parameterizable</li>
 * <li>It makes all request, request parameters, session, and servlet
 * context attributes available to templates through "Request", "RequestParameters",
 * "Session", and "Application" variables. The implementation for these variables
 * is efficient; rather than copying the attributes and parameters into {@link freemarker.template.FastHash},
 * special wrapper classes are used that implement the {@link TemplateHashModel}
 * interface.</li>
 * </ul>
 * <p>Supported initialization parameters are:</p>
 * <ul>
 * <li><strong>TemplatePath</strong> specifies the location of the templates.
 * By default, the location is the web application-relative URI of
 * the location. Alternatively, you can prepend it with <kbd>file://</kbd> to
 * indicate path in the file system (i.e. <kbd>file:///var/www/project/templates/</kbd>).
 * Note that three slashes were specified.
 * Default value is <kbd>/</kbd> (that is, the root of the web application).</li>
 * <li><strong>TemplateUpdateInterval</strong> value for update interval of the
 * template cache in seconds. Default is <code>600</code> (ten minutes)</li>
 * <li><strong>TemplateMaximumAge</strong> the maximum age of a template in the
 * cache before being expired. Default is 144 updates (~24 hours).<code>
 * <li><strong>NoCache</strong> if set to true, generates headers in the response
 * that advise the HTTP client not to cache the returned page. Default <code>false</code></li>
 * </ul>
 *
 * @author Attila Szegedi, attila@szegedi.org
 * @version $Id: FreeMarkerServlet.java,v 1.9 2003/11/16 09:58:51 run2000 Exp $
 */
public class FreeMarkerServlet extends HttpServlet
{
    public static final long serialVersionUID = -2440216393145762479L;

    private static final String INITPARAM_TEMPLATE_PATH = "TemplatePath";
    private static final String INITPARAM_TEMPLATE_DELAY = "TemplateUpdateInterval";
    private static final String INITPARAM_TEMPLATE_AGE = "TemplateMaximumAge";
    private static final String INITPARAM_NOCACHE = "NoCache";

    private static final String KEY_REQUEST = "Request";
    private static final String KEY_REQUEST_PARAMETERS = "RequestParameters";
    private static final String KEY_SESSION = "Session";
    private static final String KEY_APPLICATION = "Application";

    // Note these names start with dot, so they're essentially invisible from
    // a FM-Classic template.
    private static final String ATTR_REQUEST_MODEL = ".freemarker.Request";
    private static final String ATTR_REQUEST_PARAMETERS_MODEL = ".freemarker.RequestParameters";
    private static final String ATTR_SESSION_MODEL = ".freemarker.Session";
    private static final String ATTR_APPLICATION_MODEL = ".freemarker.Application";

    private static final String EXPIRATION_DATE;

    static
    {
        // Generate expiration date that is one year from now in the past
        GregorianCalendar expiration = new GregorianCalendar();
        expiration.roll(Calendar.YEAR, -1);
        SimpleDateFormat httpDate = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", java.util.Locale.US);
        EXPIRATION_DATE = httpDate.format(expiration.getTime());
    }

    private transient TemplateCache cache;
    /** @serial the path to templates on the file system */
    private String templatePath;
    /** @serial indicates the browser should not cache the content */
    private boolean nocache;
    /** @serial debug mode */
    protected boolean debug;
    /** @serial the update interval for Template caching, in seconds */
    protected long updateInterval;
    /** @serial the maximum age of templates before being expired from the cache */
    protected int maximumAge;

    /**
     * Initialise the servlet.
     */
    public void init()
    {
        cache = createCache();
        // Set browser caching policy
        String strnocache = getServletConfig().getInitParameter(INITPARAM_NOCACHE);
        nocache = ("true".equalsIgnoreCase(strnocache) || "yes".equalsIgnoreCase(strnocache));

        String strdebug = getServletConfig().getInitParameter("debug");
        debug = ("true".equalsIgnoreCase(strdebug) || "yes".equalsIgnoreCase(strdebug));
    }

    /**
     * Creates the {@link freemarker.template.TemplateCache} to be used for this
     * servlet.
     */
    protected TemplateCache createCache()
    {
        ServletConfig servletConfig = getServletConfig();
        templatePath = servletConfig.getInitParameter(INITPARAM_TEMPLATE_PATH);
        String strUpdateInterval = servletConfig.getInitParameter(INITPARAM_TEMPLATE_DELAY);
        String strMaximumAge = servletConfig.getInitParameter(INITPARAM_TEMPLATE_AGE);

        if(templatePath == null)
            templatePath = "/";

        // Remember, updates are measured in seconds.
        updateInterval = 10 * 60;

        try
        {
            if(strUpdateInterval != null)
                updateInterval = Long.parseLong(strUpdateInterval);
        }
        catch(NumberFormatException e)
        {
            // Intentionally ignored
        }

        // Default to expiring templates after a day
        maximumAge = 24 * 6;

        try {
            if(strMaximumAge != null)
                maximumAge = Integer.parseInt(strMaximumAge);
        } catch(NumberFormatException e)
        {
            // Intentionally ignored
        }

        // See if filesystem path is specified
        if(templatePath.startsWith("file://"))
            templatePath = templatePath.substring(7);
        // if not, resolve it according to the location of servlet context.
        else
            templatePath = getServletContext().getRealPath(templatePath);

        if(debug)
            System.err.println("TemplatePath: " + templatePath);

        FileTemplateCache cache =  new FileTemplateCache(templatePath, updateInterval);
        cache.setMaximumAge(maximumAge);
        cache.setLoadingPolicy( FileTemplateCache.LOAD_ON_DEMAND_WITH_REFRESH_CACHE );

        return cache;
    }

    /**
     * Implement the "GET" method for HTTP requests.
     *
     * @param request the servlet request
     * @param response the servlet response
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        process(request, response);
    }

    /**
     * Implement the "POST" method for HTTP requests.
     *
     * @param request the servlet request
     * @param response the servlet response
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        process(request, response);
    }

    /**
     * Process HTTP requests using FreeMarker, and send the output back to
     * the servlet container.
     *
     * @param request the servlet request
     * @param response the servlet response
     */
    private void process(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        // Give chance to subclasses to perform preprocessing
        if(preprocessRequest(request, response))
            return;

        String path = request.getPathInfo();
        if(path == null) path = "";

        if(debug)
            System.err.println("Requested template: " + path);

        Template template = (Template)cache.getItem(path);
        if(template == null)
        {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        // Set cache policy
        setBrowserCachingPolicy(response);

        TemplateModelRoot params = new FastHash();

        // Create hash model wrapper for servlet context (the application)
        ServletContext servletContext = getServletContext();
        ServletContextHashModel servletContextModel = (ServletContextHashModel)servletContext.getAttribute(ATTR_APPLICATION_MODEL);

        if(servletContextModel == null)
        {
            servletContextModel = new ServletContextHashModel(servletContext);
            servletContext.setAttribute(ATTR_APPLICATION_MODEL, servletContextModel);
            initializeServletContext(request, response);
        }

        params.put(KEY_APPLICATION, servletContextModel);

        // Create hash model wrapper for session
        HttpSession session = request.getSession();
        HttpSessionHashModel sessionModel = (HttpSessionHashModel)session.getAttribute(ATTR_SESSION_MODEL);

        if(sessionModel == null)
        {
            sessionModel = new HttpSessionHashModel(session);
            session.setAttribute(ATTR_SESSION_MODEL, sessionModel);
            initializeSession(request, response);
        }

        params.put(KEY_SESSION, sessionModel);

        // Create hash model wrapper for request
        HttpRequestHashModel requestModel = (HttpRequestHashModel)request.getAttribute(ATTR_REQUEST_MODEL);

        if(requestModel == null)
        {
            requestModel = new HttpRequestHashModel(request);
            request.setAttribute(ATTR_REQUEST_MODEL, requestModel);
        }

        params.put(KEY_REQUEST, requestModel);

        // Create hash model wrapper for request parameters
        HttpRequestParametersHashModel requestParametersModel = (HttpRequestParametersHashModel)request.getAttribute(ATTR_REQUEST_PARAMETERS_MODEL);

        if(requestParametersModel == null)
        {
            requestParametersModel = createRequestParametersHashModel(request);
            request.setAttribute(ATTR_REQUEST_PARAMETERS_MODEL, requestParametersModel);
        }

        params.put(KEY_REQUEST_PARAMETERS, requestParametersModel);

        // Give subclasses a chance to hook into preprocessing
        if(preTemplateProcess(request, response, template, params))
        {
            try
            {
                // Process the template
                template.process(params, response.getWriter());
            }
            finally
            {
                // Give subclasses a chance to hook into postprocessing
                postTemplateProcess(request, response, template, params);
            }
        }
    }

    /**
     * Called as the first step in request processing, before the templating mechanism
     * is put to work. By default does nothing and returns false. This method is
     * tipically overridden to manage serving of non-template resources (i.e. images)
     * that reside in template directory.
     * @param request the HTTP request
     * @param response the HTTP response
     * @return true to indicate this method has processed the request entirely,
     * and that the further request processing should not take place.
     */
    protected boolean preprocessRequest(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        return false;
    }

    /**
     * Retrieve the template path. This is the root of the directory structure
     * being cached by the {@link freemarker.template.TemplateCache}.
     *
     * @return the template path being cached
     */
    protected final String getTemplatePath()
    {
        return templatePath;
    }

    /**
     * Get the request parameters from the servlet as a FreeMarker hash model.
     *
     * @param request the servlet request parameters as an <code>HttpServletRequest</code>
     * @return the request parameters as a <code>TemplateHashModel</code>
     */
    protected HttpRequestParametersHashModel createRequestParametersHashModel(HttpServletRequest request)
    {
        return new HttpRequestParametersHashModel(request);
    }

    /**
     * Called when servlet detects in a request processing that
     * application-global (that is, ServletContext-specific) attributes are not yet
     * set.
     * This is a generic hook you might use in subclasses to perform a specific
     * action on first request in the context. By default it does nothing.
     * @param request the actual HTTP request
     * @param response the actual HTTP response
     */
    protected void initializeServletContext(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    }

    /**
     * Called when servlet detects in a request processing that
     * session-global (that is, HttpSession-specific) attributes are not yet
     * set.
     * This is a generic hook you might use in subclasses to perform a specific
     * action on first request in the session. By default it does nothing.
     * @param request the actual HTTP request
     * @param response the actual HTTP response
     */
    protected void initializeSession(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    }

    /**
     * Called before the execution is passed to template.process().
     * This is a generic hook you might use in subclasses to perform a specific
     * action before the template is processed. By default does nothing.
     * A typical action to perform here is to inject application-specific
     * objects into the {@link freemarker.template.TemplateModelRoot}.
     *
     * @param request the actual HTTP request
     * @param response the actual HTTP response
     * @param template the template that will get executed
     * @param data the data that will be passed to the template
     * @return true to process the template, false to suppress template processing.
     */
    protected boolean preTemplateProcess(HttpServletRequest request,
            HttpServletResponse response, Template template,
            TemplateModelRoot data) throws ServletException, IOException
    {
        return true;
    }

    /**
     * Called after the execution returns from template.process().
     * This is a generic hook you might use in subclasses to perform a specific
     * action after the template is processed. It will be invoked even if the
     * template processing throws an exception. By default does nothing.
     *
     * @param request the actual HTTP request
     * @param response the actual HTTP response
     * @param template the template that was executed
     * @param data the data that was passed to the template
     */
    protected void postTemplateProcess(HttpServletRequest request, HttpServletResponse response,
                                       Template template, TemplateModelRoot data)
        throws ServletException, IOException
    {
    }

    /**
     * If the parameter "nocache" was set to true, generate a set of headers
     * that will advise the HTTP client not to cache the returned page.
     */
    private void setBrowserCachingPolicy(HttpServletResponse response)
    {
        if(nocache)
        {
            // HTTP 1.1 browsers should defeat caching on this header
            response.setHeader("Cache-Control", "no-cache");
            // HTTP 1.0 browsers should defeat caching on this header
            response.setHeader("Pragma", "no-cache");
            // Last resort for those that ignore all of the above
            response.setHeader("Expires", EXPIRATION_DATE);
        }
    }

    /**
     * For serialization purposes, we recreate the cache when we de-serialize
     * the object.
     *
     * @param stream the input stream containing the serialized object
     */
    private void readObject(ObjectInputStream stream)
            throws IOException, ClassNotFoundException {

        stream.defaultReadObject();

        // Check whether updateInterval was serialized. In previous
        // versions of this class, it wasn't.
        if( updateInterval == 0 ) {
            // Remember, updates are measured in seconds.
            updateInterval = 10 * 60;
        }
        cache = new FileTemplateCache(templatePath, updateInterval);
    }
}
