/*
 * ====================================================================
 *
 * Copyright (c) 2000 Attila Szegedi.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes the Expose library
 *        (http://www.szegedi.org/expose)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Expose" and "Attila Szegedi" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact expose@szegedi.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package freemarker.ext.servlet;

import freemarker.template.*;

import javax.servlet.http.*;

/**
 * TemplateHashModel wrapper for <code>HttpServletRequest</code> parameters.
 *
 * @author Attila Szegedi, attila@szegedi.org
 * @author Nicholas Cull
 * @version $Id: HttpRequestParametersHashModel.java,v 1.5 2003/10/27 08:12:47 run2000 Exp $
 */
public class HttpRequestParametersHashModel implements TemplateHashModel
{
	private HttpServletRequest request;

    /**
     * Construct a new HttpRequestParameterHashModel.
     *
     * @param request the servlet request to be adapted
     */
	public HttpRequestParametersHashModel(HttpServletRequest request)
	{
		this.request = request;
	}

    /**
     * Retrieve a TemplateModel corresponding to the given request variable
     *
     * @param key the name of the request variable to be returned
     * @return a <code>TemplateModel</code> of any occurrences
     * of the variable in the request.
     */
	public TemplateModel get(String key)
	{
        String []values = request.getParameterValues(key);
        if(values == null || values.length == 0) {
            return null;
        }
        return new ParameterList(values);
	}

    /**
     * Do we have any items in the request?
     *
     * @return <code>true</code> if the request is empty, otherwise <code>false</code>
     */
	public boolean isEmpty()
	{
		return !request.getParameterNames().hasMoreElements();
	}

    /**
     * A hook for allowing <code>String</code>s to be transformed before being
     * passed into the {@link freemarker.template.TemplateModel}.
     *
     * @param string the <code>String</code> to be transcoded
     * @return the transcoded <code>String</code>
     */
	protected String transcode(String string)
	{
		return string;
	}

    /**
     * A template list model that provides access to all the values of the
     * given parameter. When used as a scalar, returns the first value.
     * When used as a list model, provides iterator and indexed access
     * to the entire set of values. Calling it as a hash with "size"
     * gives the number of elements in the list.
     *
     * @author Nicholas Cull
     */
    static class ParameterList implements TemplateScalarModel, TemplateListModel2, TemplateIndexedModel,
            TemplateHashModel {
        String []values;

        /**
         * Constructor that takes a non-empty, non-null array of String elements
         * containing all the values of the named parameter
         *
         * @param values an array of <code>String</code> values
         */
        public ParameterList( String[] values ) {
            this.values = values;
        }

        /**
         * Should never be empty, but check anyway.
         *
         * @return <code>true</code> if the array is empty, otherwise
         * <code>false</code>
         */
        public boolean isEmpty() throws TemplateModelException {
            return (values == null) || (values.length == 0);
        }

        /**
         * Returns the first <code>String</code> element in the array.
         *
         * @return the first parameter value
         */
        public String getAsString() throws TemplateModelException {
            return values[0];
        }

        /**
         * Retrieves the value at the given index.
         *
         * @param index the index at which to retrieve the value
         * @return the value at the given index
         * @throws TemplateModelException the index is out of bounds
         */
        public TemplateModel getAtIndex( long index ) throws TemplateModelException {
            if(( index < 0 ) || ( index >= values.length )) {
                throw new TemplateModelException( "Index out of bounds" );
            }
            return new FastScalar( values[ (int)index ]);
        }

        /**
         * Retrieves an iterator for walking through the underlying list.
         *
         * @return A <code>TemplateIteratorModel</code> for walking through
         * the list
         */
        public TemplateIteratorModel templateIterator() throws TemplateModelException {
            return new FastIndexedIterator( this, 0, values.length - 1 );
        }

        /**
         * Return the iterator for pooling.
         *
         * @param iterator the iterator to be released
         */
        public void releaseIterator( TemplateIteratorModel iterator ) {
        }

        /**
         * For convenience, retrieve the size of the array when the "size"
         * key is passed in.
         *
         * @param key the key "size" to retrieve the size, or any other key to
         * get a null value
         * @return the size of the array, or <code>null</code>
         */
        public TemplateModel get( String key ) throws TemplateModelException {
            if( key.equals( "size" )) {
                return new FastNumber( values.length );
            }
            return null;
        }
    }
}
