/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Iterator;

import freemarker.template.compiler.LinkedListTemplateBuilder;
import freemarker.template.compiler.StandardTemplateParser;
import freemarker.template.compiler.ParseException;
import freemarker.template.instruction.FunctionInstruction;
import freemarker.template.instruction.FunctionModel;

/**
 * <p>An application or servlet can instantiate a <code>Template</code>
 * to compile and process an HTML template.</p>
 *
 * <p>You can pass the filename of the template to the constructor, in
 * which case it is compiled immediately.  Once compiled, the template is
 * stored in an an efficient data structure for later use.</p>
 *
 * <p>To process the template and produce HTML, call the
 * {@link #process(TemplateModelRoot, Writer)} method, which takes a tree of
 * {@link TemplateModel} objects as its data model.  The root
 * node of the tree must be a {@link TemplateModelRoot}.</p>
 *
 * <p>Any error messages from exceptions thrown by the data model, or
 * generated by the <code>Template</code> during compilation or
 * processing, will be included as HTML comments in the output.</p>
 *
 * <p>To facilitate multithreading, <code>Template</code> objects are
 * immutable; if you need to recompile a template, you must make a new
 * <code>Template</code> object. In most cases, it will be sufficient
 * to let a {@link TemplateCache} do this for you.</p>
 *
 * @see TemplateCache
 * @version $Id: Template.java,v 1.41 2003/11/27 07:07:09 run2000 Exp $
 */

public class Template extends AbstractTemplate
        implements FunctionTemplateProcessor, Serializable {

    /** The root node of the compiled template. */
    protected TemplateProcessor compiledTemplate;
    /** A mapping of all function models in this compiled template. */
    protected Map functions;
    /** A cached copy of any parser exception thrown during compilation. */
    protected transient ParseException buildError;

    /** Serialization UUID for this class. */
    private static final long serialVersionUID = -2537271290895292520L;

    /**
     * Serialized form is a TemplateProcessor holding the parsed template
     * tree, followed by an array of FunctionInstructions. The array contains
     * functions to be evaluated at runtime if any functions are called.
     * This is done primarily for type correctness, and avoids serializing a
     * Map object.
     *
     * @serialField compiledTemplate TemplateProcessor object holding the
     * entire compiled template parse tree
     * @serialField functions FunctionInstruction[] an array of value
     * expressions associated with the names
     */
    private static final ObjectStreamField[] serialPersistentFields
        = { new ObjectStreamField("compiledTemplate", TemplateProcessor.class),
            new ObjectStreamField("functions", FunctionInstruction[].class) };
    /**
     * Constructs an empty template.
     */
    public Template() {
        super();
        functions = new HashMap();
    }

    /**
     * Constructs a template by compiling it from a file.  Calls
     * <code>compileFromFile()</code>.
     *
     * @param filePath the absolute path of the template file
     * to be compiled.
     */
    public Template(String filePath) throws IOException {
        super(filePath);
    }

    /**
     * Constructs a template by compiling it from a file.  Calls
     * <code>compileFromFile()</code>.
     *
     * @param file a <code>File</code> representing the
     * template file to be compiled.
     */
    public Template(File file) throws IOException {
        super(file);
    }

    /**
     * Constructs a template by compiling it from an
     * <code>InputStream</code>. Calls
     * <code>compileFromStream()</code>.
     *
     * @param stream an <tt>InputStream</tt> from which the
     * template can be read.
     */
    public Template(InputStream stream) throws IOException {
        super(stream);
    }

    /**
     * Constructs a template by compiling it from an
     * <code>Reader</code>. Calls <code>compileFromStream()</code>.
     *
     * @param stream a <code>Reader</code> from which the
     * template can be read.
     */
    public Template(Reader stream) throws IOException {
        super(stream);
    }

    /**
     * Compiles the template from an <code>Reader</code>. If the template has
     * already been compiled, this method does nothing.
     *
     * @param stream a <code>Reader</code> from which the
     * template can be read.
     */
    public void compileFromStream(Reader stream)
            throws IOException {

        synchronized(this) {
            if (compiledTemplate != null) {
                return;
            }
            if( functions == null ) {
                functions = new HashMap();
            }
        }

        StringBuffer textBuf = new StringBuffer();
        BufferedReader br = new BufferedReader( stream );
        char cbuf[] = new char[ 1024 ];
        int nSize;

        try {
            nSize = br.read( cbuf );
            while( nSize > 0 ) {
                textBuf.append( cbuf, 0, nSize );
                nSize = br.read( cbuf );
            }
        } catch (IOException e) {
            try {
                br.close();
            } catch (IOException e1) { }
            throw e;
        }

        br.close();

        TemplateProcessor newTemplate = compileText( textBuf.toString() );
        synchronized(this) {
            compiledTemplate = newTemplate;
        }
    }

    /**
     * Compiles the template text using the standard parser and builder
     * classes. To provide different compilation behavior, subclasses
     * need only override this method.
     *
     * @param text the text to compile.
     * @return a <code>TemplateProcessor</code> representing the compiled template.
     */
    protected TemplateProcessor compileText(String text) {
        StandardTemplateParser parser =
            new StandardTemplateParser(this, text);

        LinkedListTemplateBuilder builder =
            new LinkedListTemplateBuilder(this, parser);

        try {
            return builder.build();
        } catch( ParseException e ) {
            buildError = e;
            return null;
        }
    }

    /**
     * Adds a function to the template. Called by the
     * <code>TemplateBuilder</code> at compile-time.
     *
     * @param function the function to be stored by the template
     */
    public synchronized void addFunction(FunctionInstruction function) {
        functions.put(function.getName(), function);
    }

    /**
     * Retrieves a function from the template.  Called by <code>CallInstruction</code>s
     * and <code>IncludeInstruction</code>s at run-time.
     *
     * @param name the name of the function to be retrieved
     */
    public FunctionInstruction getFunction(String name) {
        return (FunctionInstruction)functions.get(name);
    }

    /**
     * Retrieve a <code>Set</code> of function names for this template.
     *
     * @return a <code>Set</code> of function names (<code>String</code> objects)
     * that have been defined for this template.
     */
    public Set getFunctionNames() {
        return functions.keySet();
    }

    /**
     * Processes the contents of this {@link TemplateProcessor} and
     * outputs the resulting text to a <code>PrintWriter</code>.
     *
     * @param modelRoot the root node of the data model.
     * @param out a <code>PrintWriter</code> to send the output to.
     * @param eventHandler a <code>TemplateEventAdapter</code> for handling any
     * events that occur during processing.
     */
    public void process(TemplateModelRoot modelRoot, PrintWriter out,
            TemplateRuntimeHandler eventHandler) {
        try {
            process(modelRoot, (Writer)out, eventHandler);
        } catch( IOException e ) {
            // Do nothing, since PrintWriter cannot throw IOException
        }
    }

    /**
     * Processes the template, using data from a template model, and outputs
     * the resulting text to a <code>PrintWriter</code>.
     *
     * @param modelRoot the root node of the data model. If <code>null</code>,
     * an empty data model is used.
     * @param out a <code>PrintWriter</code> to send the output to.
     */
    public void process(TemplateModelRoot modelRoot, PrintWriter out) {

        try {
            process(modelRoot, (Writer)out);
        } catch( IOException e ) {
            // Do nothing, since PrintWriter cannot throw IOException
        }
    }

    /**
     * Processes the template, using an empty data model, and outputs
     * the resulting text to a <code>PrintWriter</code>.
     *
     * @param out a <code>PrintWriter</code> to send the output to.
     */
    public void process(PrintWriter out) {
        try {
            process((Writer)out);
        } catch( IOException e ) {
            // Do nothing, since PrintWriter cannot throw IOException
        }
    }

    /**
     * Processes the contents of this {@link TemplateProcessor} and
     * outputs the resulting text to a <code>Writer</code>.
     *
     * @param modelRoot the root node of the data model.
     * @param out a <code>Writer</code> to send the output to.
     * @param eventHandler a <code>TemplateEventAdapter</code> for handling any
     * events that occur during processing.
     * @throws IOException an IO error occurred with the <code>Writer</code>
     * during processing
     */
    public void process(TemplateModelRoot modelRoot, Writer out,
            TemplateRuntimeHandler eventHandler) throws IOException {
        if (compiledTemplate != null) {
            if (modelRoot == null) {
                modelRoot = new FastHash();
            }
            if(functions != null) {
                copyFunctions(this, modelRoot);
            }
            compiledTemplate.process( modelRoot, out, eventHandler );
        } else if( buildError != null ) {
            eventHandler.fireExceptionThrown( this, buildError, out,
                "freemarker.template.Template.process",
                TemplateRuntimeHandler.SEVERITY_ERROR );
        }
    }

    /**
     * Processes the template, using data from a template model, and outputs
     * the resulting text to a <code>Writer</code>.
     *
     * @param modelRoot the root node of the data model.  If <code>null</code>,
     * an empty data model is used.
     * @param out a <code>Writer</code> to output the text to.
     */
    public void process(TemplateModelRoot modelRoot, Writer out)
            throws IOException {

        process(modelRoot, out, TemplateEventAdapter.DefaultEventAdapter);
    }

    /**
     * Processes the template, using an empty data model, and outputs
     * the resulting text to a <code>Writer</code>.
     *
     * @param out a <code>Writer</code> to output the text to.
     */
    public void process(Writer out) throws IOException {
        TemplateModelRoot modelRoot = new FastHash();
        process(modelRoot, out, TemplateEventAdapter.DefaultEventAdapter);
    }

    /**
     * <p>Clones the current template. A shallow clone is performed, meaning that
     * changes to the underlying structure of one will affect the structure
     * of the other. As Templates are immutable anyway, this shouldn't be
     * an issue.</p>
     *
     * <p>Cloning is used in {@link freemarker.template.cache.Cache}s, whenever
     * we need to create a new template: rather than simply creating a new
     * <code>Template</code>, we ask a {@link freemarker.template.cache.TemplateRegistry}
     * to create one for us. <code>TemplateRegistry</code> uses the
     * clone function to take an existing template, copy it, and return
     * the copy to the cache, where it is then populated.</p>
     *
     * @return a shallow clone of the current template.
     */
    public Object clone() {
        Template newTemplate = (Template)super.clone();

        if(functions != null) {
            newTemplate.functions = (Map)((HashMap)functions).clone();
        }
        return newTemplate;
    }

    /**
     * Copies functions from a template into a data model. This lets functions
     * from parent and child templates be available in both contexts.
     *
     * @param template the template from which function instructions will
     * be copied
     * @param modelRoot the template model where functions will be copied to
     */
    protected static void copyFunctions(FunctionTemplateProcessor template, TemplateModelRoot modelRoot) {
        Set functionNames = template.getFunctionNames();
        Iterator iterator = functionNames.iterator();

        while (iterator.hasNext()) {
            String functionName = (String)iterator.next();
            FunctionInstruction function = template.getFunction(functionName);
            modelRoot.put(functionName, new FunctionModel(function));
        }
    }

    /**
     * For serialization, write this object a TemplateProcessor containing
     * the compiled template, and an arrays. The array contains the
     * FunctionInstructions that may be called from the template.
     *
     * @param stream the output stream to write this object to
     */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        ObjectOutputStream.PutField fields = stream.putFields();
        int size = (functions == null) ? 0 : functions.size();
        FunctionInstruction[] functionArray = new FunctionInstruction[size];

        if( functions != null ) {
            Iterator iValues = functions.keySet().iterator();
            int i = 0;

            while(iValues.hasNext()) {
                String name = (String)iValues.next();
                functionArray[i] = (FunctionInstruction)functions.get(name);
                i++;
            }
        }

        fields.put("compiledTemplate", compiledTemplate);
        fields.put("functions", functionArray);
        stream.writeFields();
    }

    /**
     * For serialization, read this object a TemplateProcessor containing
     * the compiled template, and an array of FunctionInstructions. These
     * may be called from the template at runtime if any functions were defined
     * in the template source. Check whether the function list is null.
     *
     * @param stream the input stream to read serialized objects from
     */
    private void readObject(ObjectInputStream stream) throws IOException,
            ClassNotFoundException {

        ObjectInputStream.GetField fields = stream.readFields();

        compiledTemplate = (TemplateProcessor)fields.get("compiledTemplate", null);
        FunctionInstruction[] functionValues = (FunctionInstruction[])fields.get("functions", null);

        if(functionValues == null) {
            throw new InvalidObjectException( "Cannot create a Template with a null function list" );
        }
        functions = new HashMap();

        int size = functionValues.length;
        for(int i=0; i<size; i++) {
            FunctionInstruction functionValue = functionValues[i];
            String name = functionValue.getName();
            functions.put( name, functionValue );
        }
    }

    /**
     * Returns a string representation of the object. The value returned
     * represents the parse tree of the template.
     *
     * @return a string representation of the object.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();

        buffer.append( "FreeMarker template, " );
        buffer.append( compiledTemplate );
        return buffer.toString();
    }
}
