/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * 28 June 1999: Modified by Steve Chiu to ignore extra parentheses.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template.expression;

import freemarker.template.compiler.ParseException;
import freemarker.template.TemplateModelRoot;
import freemarker.template.FastHash;
import freemarker.template.TemplateModel;
import freemarker.template.TemplateException;

import java.util.*;

/**
 * A TemplateParser can use this class's static {@link #build}
 * method to build a complete {@link Expression} or sub-expression from a
 * <tt>List</tt> of {@link ExpressionElement}s.
 *
 * @version $Id: ExpressionBuilder.java,v 1.20 2003/11/27 07:07:12 run2000 Exp $
 */
public final class ExpressionBuilder {

    // Classes that an operator can have, arranged in precedence order
    static final int CLASS_DOT = 1;
    static final int CLASS_DYNAMIC_KEY_NAME = 2;
    static final int CLASS_METHOD_CALL = 3;

    static final int CLASS_NOT = 5;

    static final int CLASS_MULTIPLY = 6;
    static final int CLASS_DIVIDE = 7;
    static final int CLASS_MODULO = 8;

    static final int CLASS_PLUS = 9;
    static final int CLASS_MINUS = 10;

    static final int CLASS_LESS_THAN = 11;
    static final int CLASS_LESS_THAN_OR_EQUALS = 12;
    static final int CLASS_GREATER_THAN = 13;
    static final int CLASS_GREATER_THAN_OR_EQUALS = 14;

    static final int CLASS_EQUALS = 15;
    static final int CLASS_NOT_EQUALS = 16;

    static final int CLASS_AND = 17;

    static final int CLASS_OR = 18;

    /**
     * An array of arrays of operator classes, arranged in descending
     * order of precendence.
     */
    private static final int [][] opClasses = {
        { CLASS_NOT },
        { CLASS_MULTIPLY, CLASS_DIVIDE, CLASS_MODULO },
        { CLASS_PLUS, CLASS_MINUS },
        { CLASS_LESS_THAN, CLASS_LESS_THAN_OR_EQUALS, CLASS_GREATER_THAN, CLASS_GREATER_THAN_OR_EQUALS },
        { CLASS_EQUALS, CLASS_NOT_EQUALS },
        { CLASS_AND },
        { CLASS_OR }
    };

    /** An empty model used when evaluating constant expressions. */
    private static final TemplateModelRoot emptyModel = new FastHash();

    /** The number of precedence levels. */
    private static final int numLevels = opClasses.length;

    // Error messages.
    private static final String MISMATCHED_PARENS = "Mismatched parentheses";
    private static final String EMPTY_PARENS = "Empty parentheses";
    private static final String MISSING_LEFT = "Missing expression to left of operator";
    private static final String MISSING_RIGHT = "Missing expression to right of operator";
    private static final String SYNTAX_ERROR = "Syntax error in expression";
    private static final String PARSER_ERROR = "Parser error in expression";

    /**
     * Private constructor, indicating this class is not meant to be
     * instantiated.
     */
    private ExpressionBuilder() {
    }

    /**
     * Builds an {@link Expression}.
     *
     * @param elements a <code>List</code> of <code>ExpressionElement</code>s.
     * @return the complete <code>Expression</code>.
     * @throws ParseException the builder could not create a valid <code>Expression</code>
     */
    public static Expression build(List elements) throws ParseException {
        groupParens(elements);
        return buildExpression(elements);
    }

    /**
     * Builds an {@link Expression} or sub-{@link Expression}.
     *
     * @param elements a <code>List</code> of <code>ExpressionElements</code>.
     * @return the complete <code>Expression</code> or
     * sub-<code>Expression</code>.
     * @throws ParseException the builder could not create a valid <code>Expression</code>
     */
    public static Expression buildExpression(List elements)
        throws ParseException {

        // In descending order of operator precendence, associate each kind
        // of operator with its operands.
        int level = 0;
        while (elements.size() > 1 && level < numLevels) {
            associateOperators(elements, opClasses[level]);
            level++;
        }

        // The result should be a one-element list containing a
        // complete Expression.
        if (elements.size() == 1) {
            Object element = elements.get(0);

            if (element instanceof Expression) {
                Expression expression = (Expression)element;
                if (expression.isComplete()) {
                    return expression;
                } else {
                    // If the Expression isn't complete, it's
                    // because the parser gave us an operator we
                    // don't know about.
                    throw new ParseException(PARSER_ERROR);
                }
            } else {
                throw new ParseException(SYNTAX_ERROR);
            }
        } else {
            throw new ParseException(SYNTAX_ERROR);
        }
    }

    /**
     * Encapsulates parenthetical expressions by putting them in
     * sub-<code>List</code>s.
     *
     * @param elements a <code>List</code> of <code>ExpressionElements</code>.
     * @throws ParseException the builder could not create a valid <code>Expression</code>
     */
    public static void groupParens(List elements) throws ParseException {
        ListIterator iterator = elements.listIterator();
        Parenthesis paren;

        // Look for a paren.
        while ((paren = getNextParen(iterator)) != null) {

            // If the first one we find isn't an open paren, it's mismatched.
            if (paren instanceof CloseParen) {
                throw new ParseException(MISMATCHED_PARENS);
            }

            // Eat up elements and put them in a sub-List, counting
            // parens until we get to the corresponding close paren.
            iterator.remove();
            LinkedList groupedElements = new LinkedList();
            int parenLevel = 1;

            while (iterator.hasNext()) {
                Object element = iterator.next();
                iterator.remove();
                if (element instanceof OpenParen) {
                    parenLevel++;
                } else if (element instanceof CloseParen) {
                    parenLevel--;
                    if (parenLevel == 0) break;
                }
                groupedElements.add(element);
            }

            // If we didn't find it, there's a mismatch.
            if (parenLevel > 0) {
                throw new ParseException(MISMATCHED_PARENS);
            }

            // If the parenthetical expression is empty, complain.
            if (groupedElements.isEmpty()) {
                throw new ParseException(EMPTY_PARENS);
            }

            // Recurse to encapsulate any parenthetical expressions
            // in the sub-List.
            groupParens(groupedElements);

            // Insert it into the List.
            iterator.add(groupedElements);
        }
    }

    /**
     * Finds the next parenthesis.
     *
     * @param iterator an <code>Iterator</code> for the
     * <code>List</code> of <code>ExpressionElements</code>.
     * @return the next paren found, or <code>null</code> if none found.
     */
    private static Parenthesis getNextParen(ListIterator iterator) {
        while (iterator.hasNext()) {
            Object element = iterator.next();
            if (element instanceof Parenthesis) {
                return (Parenthesis)element;
            }
        }
        return null;
    }

    /**
     * Associates operators with their operands.
     *
     * @param elements a <code>List</code> of <code>ExpressionElements</code>.
     * @param expressionClasses the subclasses of <code>Expression</code> that
     * are at the current precedence level.
     * @throws ParseException the builder could not create a valid <code>Expression</code>
     */
    private static void associateOperators(List elements,
            int [] expressionClasses) throws ParseException {

        ListIterator iterator = elements.listIterator();

        // Loop through the elements.
        while (iterator.hasNext()) {
            Expression element = (Expression)iterator.next();

            if( element instanceof Operator ) {

                // If we find one of the operators we're looking for, and
                // it's incomplete, associate it with its operand(s).
                int elementClass = ((Operator)element).getOperatorClass();
                for (int i = 0; i < expressionClasses.length; i++) {
                    if (elementClass == expressionClasses[i]) {
                        if (element instanceof Binary) {
                            Binary binary = (Binary)element;
                            if (!binary.isComplete()) {
                                // Skip back over the operator.
                                iterator.previous();

                                Expression left = getPreviousExpression(iterator);

                                // Skip forward over the operator.
                                iterator.next();

                                Expression right = getNextExpression(iterator);

                                setBinaryExpression(binary, left, right, iterator);
                                break;
                            }
                        } else if (element instanceof Unary) {
                            Unary unary = (Unary)element;
                            if (!unary.isComplete()) {
                                // Associate the operator with the operand
                                // either to the left or to the right,
                                // depending on the operator's association
                                // type.
                                Expression target;
                                switch (unary.getAssociationType()) {
                                    case Unary.PREFIX:
                                        target = getNextExpression(iterator);
                                        break;
                                    case Unary.POSTFIX:
                                        // Skip back over the operator.
                                        iterator.previous();

                                        target = getPreviousExpression(iterator);

                                        // Skip forward over the operator.
                                        iterator.next();
                                        break;
                                    default:
                                        throw new ParseException(PARSER_ERROR);
                                }
                                setUnaryExpression(unary, target, iterator);
                            }
                        } else {
                            throw new ParseException(PARSER_ERROR);
                        }
                    }
                }
            }
        }
    }

    /**
     * Sets the given unary expression to the given value. The iterator
     * lies just beyond the unary instruction. If the value cannot be set,
     * an exception is raised. In addition, if the expression turns out to
     * be a constant, the expression is replaced with a constant value.
     *
     * @param unary the unary operator to be set
     * @param target the expression to set to the unary operator
     * @param iterator the current position of the iterator within the given
     * expression elements
     * @throws ParseException the target could not be set to the given unary
     * operator, or a constant expression could not be evaluated
     */
    private static void setUnaryExpression( Unary unary, Expression target,
            ListIterator iterator ) throws ParseException {

        try {
            unary.setTarget(target);
        } catch( NullPointerException e ) {
            throw new ParseException( e.getMessage(), e );
        } catch( IllegalArgumentException e ) {
            throw new ParseException( e.getMessage(), e );
        }

        if(( target.getType() & ExpressionUtils.EXPRESSION_TYPE_CONSTANT ) != 0) {
            Constant expression;

            try {
                TemplateModel outputModel = unary.getAsTemplateModel( emptyModel );
                expression = new Constant( outputModel );
            } catch( TemplateException e ) {
                throw new ParseException( "Constant expression could not be evaluated" );
            }

            // Set the expression to the new value
            iterator.previous();
            iterator.set( expression );
            iterator.next();
        }
    }

    /**
     * Sets the given unary expression to the given left- and right-hand values.
     * The iterator lies just beyond the unary instruction. If the value cannot
     * be set, an exception is raised. In addition, if the expression turns out
     * to be a constant, the expression is replaced with a constant value.
     *
     * @param binary the binary operator to be set
     * @param left the left hand expression of the binary operator
     * @param right the right hand expression of the binary operator
     * @param iterator the current position of the iterator within the given
     * expression elements
     * @throws ParseException the target could not be set to the given unary
     * operator, or a constant expression could not be evaluated
     */
    private static void setBinaryExpression( Binary binary, Expression left,
            Expression right, ListIterator iterator ) throws ParseException {

        try {
            binary.setLeft(left);
            binary.setRight(right);
        } catch( NullPointerException e ) {
            throw new ParseException( e.getMessage(), e );
        } catch( IllegalArgumentException e ) {
            throw new ParseException( e.getMessage(), e );
        }

        if(((left.getType() & ExpressionUtils.EXPRESSION_TYPE_CONSTANT) != 0 ) &&
           ((right.getType() & ExpressionUtils.EXPRESSION_TYPE_CONSTANT) != 0 )) {

            Constant expression;
            try {
                TemplateModel outputModel = binary.getAsTemplateModel( emptyModel );
                expression = new Constant( outputModel );
            } catch( TemplateException e ) {
                throw new ParseException( "Constant expression could not be evaluated" );
            }

            // Set the expression to the new value
            iterator.previous();
            iterator.set( expression );
            iterator.next();
        }
    }

    /**
     * Gets the previous {@link Expression} in the list, and removes it.
     *
     * @param iterator an <code>Iterator</code> for the
     * <code>List</code> of <code>ExpressionElements</code>.
     * @return the previous <code>Expression</code>.
     * @throws ParseException the builder could not determine a valid <code>Expression</code>
     */
    private static Expression getPreviousExpression(ListIterator iterator)
            throws ParseException {

        if (!iterator.hasPrevious()) {
            throw new ParseException(MISSING_LEFT);
        }
        Object element = iterator.previous();
        iterator.remove();
        if (!(element instanceof Expression)) {
            throw new ParseException(MISSING_LEFT);
        } else {
            return (Expression)element;
        }
    }

    /**
     * Gets the next {@link Expression} in the list, builds it if
     * necessary, and removes it.
     *
     * @param iterator an <code>Iterator</code> for the
     * <code>List</code> of <code>ExpressionElements</code>.
     * @return the next <code>Expression</code>.
     * @throws ParseException the builder could not create a valid <code>Expression</code>
     */
    private static Expression getNextExpression(ListIterator iterator)
            throws ParseException {

        if (!iterator.hasNext()) {
            throw new ParseException(MISSING_RIGHT);
        }

        Object element = iterator.next();
        iterator.remove();

        if (element instanceof Expression) {
            return (Expression)element;
        } else if (element instanceof List) {
            return buildExpression((List)element);
        } else {
            throw new ParseException(MISSING_RIGHT);
        }
    }

    /**
     * Builds a complete {@link Variable}.
     *
     * @param elements a <code>List</code> of <code>ExpressionElements</code>.
     * @return the complete <code>Variable</code>.
     * @throws ParseException the builder could not create a valid <code>Variable</code>
     */
    public static Expression buildVariable(List elements)
            throws ParseException {

        Expression expression;
        int size = elements.size();

        // Associate each kind of operator with its operands.
        if( size > 1 ) {
            expression = associateVariableOperators( elements );
        } else if( size == 1 ) {
            expression = (Expression)elements.get( 0 );
        } else {
            throw new ParseException( SYNTAX_ERROR );
        }

        if( expression.isComplete() ) {
            return expression;
        } else {
            // If the Expression isn't complete, it's
            // because the parser gave us an operator we
            // don't know about.
            throw new ParseException( PARSER_ERROR );
        }
    }

    /**
     * Associates {@link Variable} operators with their operands. This takes advantage
     * of the fact that all {@link Variable} operators are postfix unary operators.
     *
     * @param elements a <code>List</code> of <code>ExpressionElements</code>.
     * @return The correctly associated variable
     * @throws ParseException something went wrong during the association
     */
    private static Expression associateVariableOperators( List elements )
            throws ParseException {

        ListIterator iterator = elements.listIterator();
        Expression out;
        Object element = iterator.next();

        if( element instanceof Variable ) {
            out = (Expression)element;
        } else {
            throw new ParseException( PARSER_ERROR );
        }

        // Loop through the elements.
        while ( iterator.hasNext() ) {
            element = iterator.next();

            // If we find one of the operators we're looking for
            // associate it with its operand.
            if( element instanceof Unary ) {
                Unary unary = (Unary)element;
                    try {
                        unary.setTarget(out);
                    } catch( NullPointerException e ) {
                        throw new ParseException( e.getMessage(), e );
                    } catch( IllegalArgumentException e ) {
                        throw new ParseException( e.getMessage(), e );
                    }
                out = unary;
            } else {
                throw new ParseException( PARSER_ERROR );
            }
        }
        return out;
    }
}
