/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template.expression;

import java.util.WeakHashMap;
import java.io.*;

import freemarker.template.*;

/**
 * An identifer in a variable. Can be used stand-alone to represent a
 * variable in the root model, or combined with the {@link Dot} operator
 * to represent models contained within hash models.
 *
 * @version $Id: Identifier.java,v 1.25 2003/11/27 11:36:45 run2000 Exp $
 */
public final class Identifier implements Variable, Serializable {
    /**
     * The name for this identifier.
     *
     * @serial the name of the identifier
     */
    private final String name;

    /**
     * A weak map of Strings that is used for storing canonical template
     * identifiers. We store this in a weak hash map so that they can be
     * eventually reclaimed by the garbage collector in the event that the
     * template expires.
     */
    private static final WeakHashMap identifierCache = new WeakHashMap();

    /** Serialization UUID for this class. */
    private static final long serialVersionUID = 1053885113751957584L;

    /**
     * Retrieve a canonical instance of the identifier for the given name.
     * A weak hash map is used as a backing store, so identifiers are
     * unique only when garbage collection hasn't caused the map to be
     * reclaimed.
     *
     * @param name the name of the identifier
     * @return a new <code>Identifier</code> for the given name
     * @throws IllegalArgumentException Identifier name was null
     */
    public static Identifier getInstance( String name ) {
        if( name == null ) {
            throw new IllegalArgumentException( "Identifier cannot be null" );
        }

        Identifier interned = (Identifier)identifierCache.get( name );

        if( interned == null ) {
            String uniqueName = new String( name );
            interned = new Identifier( uniqueName );
            identifierCache.put( uniqueName,  interned );
        }
        return interned;
    }

    /**
     * Constructs a new <code>Identifier</code> with the given name.
     *
     * @param name the name of the identifier
     */
    private Identifier(String name) {
        this.name = name;
    }

    /**
     * Retrieve the name of the identifier.
     *
     * @return the name of the identifier
     */
    public String getName() {
        return name;
    }

    /**
     * Retrieve the name of this <code>Identifier</code>.
     *
     * @param modelRoot the <code>TemplateModelRoot</code> used to evaluate
     * the name of the identifier
     * @return the name of the identifier
     */
    public String getName(TemplateModelRoot modelRoot) {
        return name;
    }


    /**
     * Has the identifier's name been assigned?
     *
     * @return <code>true</code> if the identifier is named, otherwise <code>false</code>
     */
    public boolean isComplete() {
        return true;
    }

    /**
     * Determine the type of result that can be calculated by this expression.
     * This is in the form of an integer constant ored together from
     * values in the {@link ExpressionUtils} class.
     */
    public int getType() {
        return ExpressionUtils.EXPRESSION_TYPE_VARIABLE;
    }

    /**
     * Gets the identifier's referent in modelRoot.
     *
     * @param modelRoot the root node of the data model.
     * @return the <code>TemplateModel</code> that the variable refers to.
     */
    public TemplateModel getAsTemplateModel(TemplateModelRoot modelRoot)
            throws TemplateException {

        try {
            return modelRoot.get( name );
        } catch( TemplateModelException e ) {
            throw new TemplateException("Couldn't get referent of " +
                        name, e );
        }
    }

    /**
     * Returns the name of the identifier as a <code>String</code>.
     *
     * @return the name of the identifier.
     */
    public String toString() {
        return name;
    }

    /**
     * Tests the equality of two <code>Identifier</code>s.
     *
     * @return <code>true</code> if the identifiers are equal, otherwise <code>false</code>
     */
    public boolean equals(Object o) {
        if(! (o instanceof Identifier)) {
            return false;
        }

        final Identifier id = (Identifier)o;
        if( name == null ) {
            return (id.name == null);
        }
        return name.equals(id.name);
    }

    /**
     * Returns a hash code value for the <code>Identifier</code>.
     *
     * @return  a hash code value for this Identifier.
     */
    public int hashCode() {
        if (name == null) {
            return 0;
        }
        return name.hashCode() + 17;
    }

    /**
     * For serialization purposes, resolve a deserialized instance to an
     * instance in the identifier cache, if possible. If no entry exists,
     * just return the current instance without adding it to the cache.
     */
    private Object readResolve() throws ObjectStreamException {
        Identifier cachedIdentifier = (Identifier)identifierCache.get( name );

        if( cachedIdentifier != null ) {
            return cachedIdentifier;
        }
        return this;
    }

    /**
     * For serialization, read this object normally, then check whether the
     * name has been deserialized to null.
     *
     * @param stream the input stream to read serialized objects from
     */
    private void readObject(ObjectInputStream stream) throws IOException,
            ClassNotFoundException {
        stream.defaultReadObject();
        if( name == null ) {
            throw new InvalidObjectException( "Cannot create an Identifier with a null name" );
        }
    }
}
