/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template.instruction;

import freemarker.template.*;
import freemarker.template.compiler.TemplateBuilder;
import freemarker.template.expression.Expression;

import java.io.IOException;
import java.io.Writer;
import java.io.Serializable;

/**
 * Represents a case in a switch statement. Unexpectedly, this is a superclass
 * for the regular {@link CaseInstruction} class.
 *
 * @version $Id: DefaultCaseInstruction.java,v 1.9 2003/11/27 11:36:45 run2000 Exp $
 * @see CaseInstruction
 * @see SwitchInstruction
 */
public class DefaultCaseInstruction implements TemplateProcessor, Instruction,
        Serializable {

    /**
     * The template body to process if the default case is reached.
     *
     * @serial the template body to be processed if the case matches
     */
    protected TemplateProcessor body;

    /** Serialization UUID for this class. */
    private static final long serialVersionUID = -7534875993650505782L;

    /** Default constructor. */
    public DefaultCaseInstruction() { }

    /**
     * Retrieves the {@link Expression} to be
     * evaluated when the <code>case</code> instruction is encountered.
     *
     * @return the <code>Expression</code> associated with this
     * <code>CaseInstruction</code>
     */
    public Expression getExpression() {
        // The actual expression returned is irrelevant, since it is
        // never tested for default cases.
        return null;
    }

    /**
     * Is this the default case?
     *
     * @return <code>true</code> if this is the default case,
     * otherwise <code>false</code>
     */
    public boolean isDefault() {
        return true;
    }

    /**
     * Sets the body to be processed if the case expression evaluated to
     * <code>true</code>.
     *
     * @param body the <code>TemplateProcessor</code> to be processed if this
     * <code>Case</code> passes the test.
     */
    public final void setBody(TemplateProcessor body) {
        this.body = body;
    }

    /**
     * Sets whether this case instruction contains a break instruction.
     *
     * @param hasBreak <code>true</code> if this <code>CaseInstruction</code>
     * has a <code>break</code>, otherwise <code>false</code>
     */
    public void setBreak(boolean hasBreak) {
        // Do nothing, since for Default cases this is irrelevant
    }

    /**
     * Does this case instruction contain a break instruction?
     *
     * @return <code>true</code> if this <code>Case</code> has a <code>break</code>
     * instruction, otherwise <code>false</code>
     */
    public boolean hasBreak() {
        return true;
    }

    /**
     * Is this an end instruction?
     *
     * @return <code>true</code>, to indicate this is an end instruction
     */
    public final boolean isEndInstruction() {
        return true;
    }

    /**
     * Retrieve the type of end instruction, if any.
     *
     * @return <code>DEFAULT</code>
     */
    public int getEndType() {
        return DEFAULT;
    }

    /**
     * A {@link TemplateBuilder} can call
     * this method to have an <code>Instruction</code> call it back to be built.
     * For end instructions, there is nothing more to be build, so return
     * immediately.
     *
     * @param builder the builder to be called back by this method
     */
    public final TemplateProcessor callBuilder(TemplateBuilder builder) {
        return this;
    }

    /**
     * Process this <code>&lt;default&gt;</code> instruction.
     *
     * @param modelRoot the root node of the data model.
     * @param out a <code>Writer</code> to send the output to.
     * @param eventHandler a <code>TemplateEventAdapter</code> for handling any
     * events that occur during processing.
     */
    public final void process(TemplateModelRoot modelRoot, Writer out,
            TemplateRuntimeHandler eventHandler) throws IOException {
        if (body != null) {
            body.process(modelRoot, out, eventHandler);
        }
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a <code>String</code> representing this instruction subtree
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append( "default " );
        buffer.append( body );
        return buffer.toString();
    }
}
