/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template.instruction;

import java.io.*;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import freemarker.template.*;
import freemarker.template.expression.Identifier;

/**
 * <p>An immutable {@link freemarker.template.TemplateModel} wrapper for
 * a {@link FunctionInstruction}s. This approach allows us to add functions
 * to the data model, while still enforcing the restriction that every
 * element of the data model should implement the
 * {@link freemarker.template.TemplateModel} interface.</p>
 *
 * <p>Notably, this class is a direct subclass of
 * {@link freemarker.template.TemplateModel}, rather than going through an
 * intermediate interface.</p>
 *
 * @version $Id: FunctionModel.java,v 1.19 2003/11/27 11:36:45 run2000 Exp $
 */
public final class FunctionModel implements TemplateModel, Serializable {
    /**
     * @serial the function instruction to be evaluated when this model
     * is called
     */
    private final FunctionInstruction function;

    /** Serialization UUID for this class. */
    private static final long serialVersionUID = -3155379731690082587L;

    /**
     * Constructor for the function model.
     *
     * @param function the function to be stored in the model
     * @throws NullPointerException the function is null
     */
    public FunctionModel(FunctionInstruction function) {
        if( function == null ) {
            throw new NullPointerException( "The function instruction cannot be null" );
        }
        this.function = function;
    }

    /**
     * Is the function model populated?
     *
     * @return <code>true</code> if there is no function instruction
     * associated with the model, otherwise <code>false</code>
     */
    public boolean isEmpty() {
        return false;
    }

    /**
     * Calls the specified {@link FunctionInstruction}.
     *
     * @param modelRoot the root of the template model
     * @param out the output stream to send the results
     * @param eventHandler handler for any events fired
     * @param argValues the arguments passed to the function
     * @throws java.io.IOException there was an IO error writing the results
     */
    public void callFunction(TemplateModelRoot modelRoot,  Writer out,
            TemplateRuntimeHandler eventHandler, List argValues) throws IOException {

        // Test whether we have a matching number of arguments
        List argumentNames = function.getArgumentNames();
        if (argumentNames.size() != argValues.size()) {
            eventHandler.fireExceptionThrown(this, new TemplateException(
                    "Wrong number of arguments in function call"
                    ), out,
                    "freemarker.template.instruction.CallInstruction.process",
                    TemplateRuntimeHandler.SEVERITY_WARNING);
            return;
        }

        // Put the arguments in the data model under the argument names,
        // temporarily hiding the current values of those names.
        Map oldValues = new HashMap();
        Iterator argNamesIterator = argumentNames.iterator();
        Iterator argIterator = argValues.iterator();
        while( argIterator.hasNext() ) {
            Identifier argName = (Identifier)argNamesIterator.next();

            // Hide the current value of whatever has this name.
            TemplateModel oldReferent = null;
            try {
                oldReferent = argName.getAsTemplateModel( modelRoot );
            } catch (TemplateException te) {
                eventHandler.fireExceptionThrown(this, new TemplateException(
                    "Couldn't hide model " + argName.getName(), te )
                    , out,
                    "freemarker.template.instruction.CallInstruction.process",
                    TemplateRuntimeHandler.SEVERITY_WARNING);
            }

            if (oldReferent != null) {
                oldValues.put(argName.getName(), oldReferent);
            }

            // Put the argument's value in the data model.
            TemplateModel argModel = (TemplateModel)argIterator.next();
            modelRoot.put(argName.getName(), argModel);
        }

        // Call the function.
        function.process(modelRoot, out, eventHandler);

        // Restore the old values of the argument names.
        argNamesIterator = argumentNames.iterator();
        while (argNamesIterator.hasNext()) {
            Identifier argName = (Identifier)argNamesIterator.next();
            String name = argName.getName();
            TemplateModel oldReferent = (TemplateModel)oldValues.get(name);
            if (oldReferent != null) {
                modelRoot.put(name, oldReferent);
            } else {
                modelRoot.remove(name);
            }
        }
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a <code>String</code> representing this instruction subtree
     */
    public String toString() {
        return function.toString();
    }

    /**
     * For serialization, read this object normally, then check whether the
     * function instruction is null.
     *
     * @param stream the input stream to read serialized objects from
     */
    private void readObject(ObjectInputStream stream) throws IOException,
            ClassNotFoundException {
        stream.defaultReadObject();
        if( function == null ) {
            throw new InvalidObjectException( "Cannot create a FunctionModel with a null instruction" );
        }
    }
}
