/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.template.instruction;

import freemarker.template.*;
import freemarker.template.expression.Variable;
import freemarker.template.expression.Identifier;
import freemarker.template.expression.Expression;
import freemarker.template.expression.ExpressionUtils;

import java.io.IOException;
import java.io.Writer;
import java.io.ObjectInputStream;
import java.io.InvalidObjectException;

/**
 * An instruction that processes a {@link freemarker.template.TemplateListModel}
 * or {@link freemarker.template.TemplateListModel2}.
 * This can be either in the form of a <code>&lt;list ... as ...&gt;</code>
 * instruction or a <code>&lt;foreach ... in ...&gt;</code> instruction.
 *
 * @version $Id: ListInstruction.java,v 1.29 2003/11/27 11:36:45 run2000 Exp $
 */
public final class ListInstruction extends GenericStartInstruction {
    /**
     * @serial the expression that evaluates to a template list model
     */
    private final Expression listExpression;
    /**
     * @serial the variable used to index over the list
     */
    private final Identifier indexVariable;
    private transient Identifier iteratorVariable;

    /** Serialization UUID for this class. */
    private static final long serialVersionUID = -5366066980827389242L;

    /**
     * Constructs a list/foreach instruction containing the list variable
     * and the index variable.
     *
     * @param listExpression a variable or list expression referring to a
     * <code>TemplateListModel</code>.
     * @param indexVariable an arbitrary index variable.
     * @throws NullPointerException list expression or index variable are null
     * @throws IllegalArgumentException the list expression wasn't a list
     */
    public ListInstruction(Expression listExpression, Identifier indexVariable) {
        if( indexVariable == null ) {
            throw new NullPointerException( "Index variable in list instruction cannot be null" );
        }
        if(( listExpression.getType() & ExpressionUtils.EXPRESSION_TYPE_LIST ) == 0 ) {
            throw new IllegalArgumentException( "List instruction must contain a list" );
        }
        this.listExpression = listExpression;
        this.indexVariable = indexVariable;
        iteratorVariable = Identifier.getInstance( indexVariable.getName() + "#" );
    }

    /**
     * Is this the right kind of instruction for the given {@link EndInstruction}?
     *
     * @param endInstruction the end instruction we're testing
     * @return <code>true</code> if the <code>EndInstruction</code> is a
     * list end instruction, otherwise <code>false</code>
     */
    public boolean testEndInstruction(Instruction endInstruction) {
        return (endInstruction.getEndType() == LIST_END);
    }

    /**
     * Processes the <code>&lt;list ... &gt;</code> or
     * <code>&lt;foreach ... &gt;</code> instruction.
     *
     * @param modelRoot the root node of the data model.
     * @param out a <code>Writer</code> to send the output to.
     * @param eventHandler a <code>TemplateEventAdapter</code> for handling any
     * events that occur during processing.
     */
    public void process(TemplateModelRoot modelRoot, Writer out,
            TemplateRuntimeHandler eventHandler) throws IOException {

        if (body == null) {
            return;
        }

        String listVariableName;
        try {
            if (listExpression instanceof Variable) {
                listVariableName = ((Variable)listExpression).getName(modelRoot);
            } else {
                listVariableName = "list literal";
            }
        } catch (TemplateException te) {
            eventHandler.fireExceptionThrown(this, new TemplateException(
                    "Couldn't resolve name of list variable", te ),
                    out,
                    "freemarker.template.instruction.ListInstruction.process",
                    TemplateRuntimeHandler.SEVERITY_WARNING);
            return;
        }

        TemplateModel baseModel;
        try {
            baseModel = listExpression.getAsTemplateModel(modelRoot);
        } catch (TemplateException te) {
            eventHandler.fireExceptionThrown(this, new TemplateException(
                    "Couldn't get referent of " + listVariableName,
                    te ),
                    out,
                    "freemarker.template.instruction.ListInstruction.process",
                    TemplateRuntimeHandler.SEVERITY_WARNING);
            return;
        }

        try {
            if ((baseModel == null) || (baseModel.isEmpty())) {
                return;
            }
        } catch (TemplateModelException e) {
            eventHandler.fireExceptionThrown(this, new TemplateException(
                "Couldn't perform list instruction on " +
                listVariableName, e ),
                out,
                "freemarker.template.instruction.ListInstruction.process",
                TemplateRuntimeHandler.SEVERITY_WARNING);
            return;
        }

        String iteratorName = iteratorVariable.getName();

        // Hide old value of iterator variable, if any
        TemplateModel oldIteratorReferent = null;
        try {
            oldIteratorReferent = iteratorVariable.getAsTemplateModel(modelRoot);
        } catch (TemplateException te) { }

        String indexName = indexVariable.getName();

        // Hide the old value of the loop index variable.
        TemplateModel oldIndexReferent = null;
        try {
            oldIndexReferent = indexVariable.getAsTemplateModel(modelRoot);
        } catch (TemplateException te) { }

        try {
            if (baseModel instanceof TemplateListModel2) {

                TemplateListModel2 baseListModel = (TemplateListModel2)baseModel;
                TemplateIteratorModel iteratorModel = null;

                try {
                    iteratorModel = baseListModel.templateIterator();

                    // If no iterator was returned, we can finish immediately,
                    // avoiding any cleanup.
                    if(( iteratorModel == null ) || ( iteratorModel.isEmpty() )) {
                        return;
                    }

                    // Set the iterator to be part of the model, in case the iterator
                    // is a multi-model.
                    modelRoot.put(iteratorName, iteratorModel);

                    // Loop, setting the index to refer to each element in turn.
                    TemplateModel currElement;
                    while( iteratorModel.hasNext() ) {
                        currElement = iteratorModel.next();
                        modelRoot.put(indexName, currElement);
                        body.process(modelRoot, out, eventHandler);
                    }
                } finally {
                    if( iteratorModel != null ) {
                        baseListModel.releaseIterator(iteratorModel);
                    }
                }
            } else if (baseModel instanceof TemplateListModel) {
                TemplateListModel baseListModel = (TemplateListModel)baseModel;

                // Set the iterator to be part of the model, in case the iterator
                // is a multi-model. For TemplateListModel, the iterator is
                // the list itself, but we do it anyway for transparency between
                // list models.
                modelRoot.put(iteratorName, baseModel);

                // Loop, setting the index to refer to each element in turn.
                TemplateModel currElement;
                if (!baseListModel.isRewound()) {
                    baseListModel.rewind();
                }
                while (baseListModel.hasNext()) {
                    currElement = baseListModel.next();
                    modelRoot.put(indexName, currElement);
                    body.process(modelRoot, out, eventHandler);
                }
            } else if (baseModel instanceof TemplateScalarModel) {
                // Treat a scalar as a list with one element.
                modelRoot.put(iteratorName, baseModel);
                modelRoot.put(indexName, baseModel);
                body.process(modelRoot, out, eventHandler);
            } else {
                eventHandler.fireExceptionThrown(this, new TemplateException(
                    listVariableName +
                    " is not a TemplateListModel, TemplateListModel2, or a TemplateScalarModel"
                    ), out,
                    "freemarker.template.instruction.ListInstruction.process",
                    TemplateRuntimeHandler.SEVERITY_WARNING);
            }
        } catch (TemplateModelException e) {
            eventHandler.fireExceptionThrown(this, new TemplateException(
                "Couldn't read from list " + listVariableName, e ),
                out,
                "freemarker.template.instruction.ListInstruction.process",
                TemplateRuntimeHandler.SEVERITY_WARNING);
        } finally {
            // Restore the old value of the loop index variable, if there was one.
            if (oldIndexReferent != null) {
                modelRoot.put(indexName, oldIndexReferent);
            } else {
                modelRoot.remove(indexName);
            }
            // Restore the old value of the iterator variable, if there was one.
            if (oldIteratorReferent != null) {
                modelRoot.put(iteratorName, oldIteratorReferent);
            } else {
                modelRoot.remove(iteratorName);
            }
        }
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a <code>String</code> representing this instruction subtree
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append( "list " );
        buffer.append( listExpression );
        buffer.append( " as " );
        buffer.append( indexVariable );
        buffer.append( ' ' );
        buffer.append( body );
        return buffer.toString();
    }

    /**
     * When deserializing this object, read the object normally, test whether
     * the list expression or index variable is null, then recreate the
     * transient iterator field.
     *
     * @param stream the input stream to read serialized objects from
     */
    private void readObject(ObjectInputStream stream)
            throws IOException, ClassNotFoundException {

        stream.defaultReadObject();
        if(( listExpression == null ) || ( indexVariable == null )) {
            throw new InvalidObjectException( "Cannot create a ListInstruction with a null expression or index variable" );
        }
        iteratorVariable = Identifier.getInstance( indexVariable.getName() + "#" );
    }
}
