/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.testcase;

import java.util.List;
import java.util.ArrayList;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestResult;

import freemarker.template.*;
import freemarker.testcase.models.ListModel1;
import freemarker.testcase.models.ListModel2;
import freemarker.ext.misc.ExtendedList;

/**
 * A simple test case for performing unit tests on ExtendedList.
 * These complement the tests performed by the other list tests
 * by improving code coverage for this model in particular.
 *
 * @author Nicholas Cull
 * @version $Id: TestListModel2.java,v 1.2 2003/11/28 01:17:07 run2000 Exp $
 */
public final class TestListModel2 extends TestCase {

    /**
     * Constructor that passes the name of the test up to the JUnit
     * constructor.
     *
     * @param aTestname the name of the test
     */
    public TestListModel2( String aTestname ) {
        super( aTestname );
    }

    /**
     * Performs an ExtendedList test.
     */
    public void testSimpleArrayList() throws Exception {

        // Step 1: create an object model to test
        TemplateModel[] model = new TemplateModel[3];

        model[0] = new SimpleScalar( "test1" );
        model[1] = new SimpleScalar( "test2" );
        model[2] = new SimpleNumber( new Long( 3 ));
        ExtendedList extendedList = new ExtendedList( model );

        // Step 2: assert that the model is correct
        TemplateScalarModel stringModel = (TemplateScalarModel)extendedList.getAtIndex( 0 );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test1" );
        TemplateModel compareModel = new SimpleScalar( "test1" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 1 );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test2" );
        compareModel = new SimpleScalar( "test2" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        TemplateNumberModel numberModel = (TemplateNumberModel)extendedList.getAtIndex( 2 );
        assertNotNull( numberModel );
        assertTrue( "Expecting a SimpleNumber", numberModel instanceof SimpleNumber );
        long number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 3, number );
        compareModel = new SimpleNumber();
        ((SimpleNumber)compareModel).setValue("3");
        assertEquals( "SimpleNumbers should be equal", numberModel, compareModel );
        assertEquals( "SimpleNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );
    }

    /**
     * Performs an ExtendedList test.
     */
    public void testSimpleCollectionList() throws Exception {

        // Step 1: create an object model to test
        List model = new ArrayList(3);

        model.add( new SimpleScalar( "test1" ));
        model.add( new SimpleScalar( "test2" ));
        model.add( new SimpleNumber( new Long( 3 )));
        ExtendedList extendedList = new ExtendedList( model );

        // Step 2: assert that the model is correct
        TemplateScalarModel stringModel = (TemplateScalarModel)extendedList.getAtIndex( 0 );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test1" );
        TemplateModel compareModel = new SimpleScalar( "test1" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 1 );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test2" );
        compareModel = new SimpleScalar( "test2" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        TemplateNumberModel numberModel = (TemplateNumberModel)extendedList.getAtIndex( 2 );
        assertNotNull( numberModel );
        assertTrue( "Expecting a SimpleNumber", numberModel instanceof SimpleNumber );
        long number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 3, number );
        compareModel = new SimpleNumber();
        ((SimpleNumber)compareModel).setValue(3);
        assertEquals( "SimpleNumbers should be equal", numberModel, compareModel );
        assertEquals( "SimpleNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );
    }

    /**
     * Performs an ExtendedList test.
     */
    public void testListCopy1() throws Exception {
        ListModel1 model1 = new ListModel1();

        // Step 1: create an object model to test
        ExtendedList extendedList = new ExtendedList( model1 );

        // Step 2: assert that the model is correct
        assertTrue( "ExtendedList should not be empty", ! extendedList.isEmpty() );

        TemplateScalarModel stringModel = (TemplateScalarModel)extendedList.getAtIndex( 0 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "one" );
        TemplateModel compareModel = new FastScalar( "one" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 1 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "two" );
        compareModel = new FastScalar( "two" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 2 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "three" );
        compareModel = new FastScalar( "three" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 3 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "four" );
        compareModel = new FastScalar( "four" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 4 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "five" );
        compareModel = new FastScalar( "five" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        extendedList.clear();
        assertTrue( "ExtendedList should be empty", extendedList.isEmpty() );
    }

    /**
     * Performs an ExtendedList test.
     */
    public void testListCopy2() throws Exception {
        ListModel2 model1 = new ListModel2();

        // Step 1: create an object model to test
        ExtendedList extendedList = new ExtendedList( model1 );

        // Step 2: assert that the model is correct
        assertTrue( "ExtendedList should not be empty", ! extendedList.isEmpty() );

        TemplateScalarModel stringModel = (TemplateScalarModel)extendedList.getAtIndex( 0 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "one" );
        TemplateModel compareModel = new FastScalar( "one" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 1 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "two" );
        compareModel = new FastScalar( "two" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 2 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "three" );
        compareModel = new FastScalar( "three" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 3 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "four" );
        compareModel = new FastScalar( "four" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 4 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "five" );
        compareModel = new FastScalar( "five" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        extendedList.clear();
        assertTrue( "ExtendedList should be empty", extendedList.isEmpty() );
    }

    /**
     * Performs an ExtendedList test.
     */
    public void testListAdd() throws Exception {

        // Step 1: create an object model to test
        ExtendedList extendedList = new ExtendedList();

        extendedList.add( "test1" );
        extendedList.add( new FastScalar( "test2" ));
        extendedList.add( 3 );
        extendedList.add( new Long( 4 ));
        extendedList.add( true );

        // Step 2: assert that the model is correct
        assertTrue( "ExtendedList should not be empty", ! extendedList.isEmpty() );

        TemplateScalarModel stringModel = (TemplateScalarModel)extendedList.getAtIndex( 0 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test1" );
        TemplateModel compareModel = new FastScalar( "test1" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 1 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "test2" );
        compareModel = new FastScalar( "test2" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        TemplateNumberModel numberModel = (TemplateNumberModel)extendedList.getAtIndex( 2 );
        assertNotNull( numberModel );
        assertTrue( "Expecting a FastNumber", numberModel instanceof FastNumber );
        long number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 3, number );
        compareModel = new FastNumber( 3 );
        assertEquals( "FastNumbers should be equal", numberModel, compareModel );
        assertEquals( "FastNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        numberModel = (TemplateNumberModel)extendedList.getAtIndex( 3 );
        assertNotNull( numberModel );
        assertTrue( "Expecting a FastNumber", numberModel instanceof FastNumber );
        number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 4, number );
        compareModel = new FastNumber( 4 );
        assertEquals( "FastNumbers should be equal", numberModel, compareModel );
        assertEquals( "FastNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)extendedList.getAtIndex( 4 );
        assertNotNull( numberModel );
        assertTrue( "Expecting a FastBoolean", stringModel instanceof FastBoolean );
        assertTrue( "FastBoolean should be true", ! stringModel.isEmpty() );
        assertEquals( "FastBooleans should be equal", stringModel, FastBoolean.TRUE );
        assertEquals( "FastBooleans hash codes should match", stringModel.hashCode(), FastBoolean.TRUE.hashCode() );
        assertTrue( "FastBooleans should be the same object", stringModel == FastBoolean.TRUE );

    }

    /**
     * Performs a test of shelf life in ExtendedList.
     */
    public void testShelfLife() throws Exception {
        ExtendedList list = new ExtendedList();

        list.add( "test" );

        assertTrue( "ExtendedList should be permanent by default", list.isPermanent() );
        assertEquals( "Shelf life should be 0 by default", list.getShelfLife(), 0 );

        // Shelf life is ~ one minute
        list.setShelfLife( 1 );

        assertEquals( "Shelf life should same as set", list.getShelfLife(), 1 );
        assertTrue( "Shelf life should not be stale yet", ! list.isStale() );
        assertTrue( "List should not be permanent currently", ! list.isPermanent() );

        list.setPermanent();

        assertTrue( "Shelf life should be permanent again", list.isPermanent() );
        assertEquals( "Shelf life should still be 1 minute", list.getShelfLife(), 1 );

        ExtendedList list2 = new ExtendedList( list );

        assertEquals( "Extended lists should be equal", list, list2 );
        assertEquals( "Hash codes should match", list.hashCode(), list2.hashCode() );
    }

    /**
     * Return a suite of all the tests in this class by reflection.
     *
     * @return a <code>TestSuite</code> of all the methods starting with the
     * word "test"
     */
    public static Test suite() {
        return new TestSuite(TestListModel2.class);
    }

    /**
     * Bootstrap for the self-test code.
     */
    public static void main( String[] argc ) throws Exception {
        TestListModel2 cTest = new TestListModel2( "testListModel" );
        TestResult cResults = new TestResult();

        cTest.run( cResults );
    }
}
