/*
 * FreeMarker: a tool that allows Java programs to generate HTML
 * output using templates.
 * Copyright (C) 1998-2003 Benjamin Geer
 * Email: beroul@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package freemarker.testcase;

import junit.framework.TestCase;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import freemarker.template.*;

/**
 * A simple test case for serializing and deserializing template models.
 * In addition, it performs tests of equality using the <code>equals</code>
 * and <code>hashCode</code> methods.
 *
 * @author Nicholas Cull
 * @version $Id: TestSerialModels.java,v 1.9 2003/11/09 03:18:26 run2000 Exp $
 */
public final class TestSerialModels extends TestCase {

    /**
     * Constructor that passes the name of the test up to the JUnit
     * constructor.
     *
     * @param aTestname the name of the test
     */
    public TestSerialModels( String aTestname ) {
        super( aTestname );
    }

    /**
     * Make a new list model. This is used both for serialization and later
     * testing of <code>equals</code> and <code>hashCode</code> methods.
     *
     * @return a new SimpleList for serialization and comparison
     */
    private SimpleList makeList1() {
        SimpleList list1 = new SimpleList();
        list1.add( "Scalar4" );
        list1.add( "Scalar5" );
        return list1;
    }

    /**
     * Make a new list model. This is used both for serialization and later
     * testing of <code>equals</code> and <code>hashCode</code> methods.
     *
     * @return a new FastList for serialization and comparison
     */
    private FastList makeList2() {
        ArrayList aList = new ArrayList();
        aList.add( new FastScalar( "Scalar6" ));
        aList.add( new FastScalar( "Scalar7" ));

        FastList list2 = new FastList( aList );
        return list2;
    }

    /**
     * Make a new hash model. This is used both for serialization and later
     * testing of <code>equals</code> and <code>hashCode</code> methods.
     *
     * @return a new SimpleHash for serialization and comparison
     */
    private SimpleHash makeHash1() {
        SimpleHash hash1 = new SimpleHash();
        hash1.put( "string10", "Scalar10" );
        hash1.put( "string11", new FastScalar( "Scalar11" ));
        hash1.put( "string12", new SimpleScalar( "Scalar12" ));
        return hash1;
    }

    /**
     * Make a new hash model. This is used both for serialization and later
     * testing of <code>equals</code> and <code>hashCode</code> methods.
     *
     * @return a new FastHash for serialization and comparison
     */
    private FastHash makeHash2() {
        HashMap aMap = new HashMap();
        aMap.put( "string13", new FastScalar( "Scalar13" ));
        aMap.put( "string14", new SimpleScalar( "Scalar14" ));
        aMap.put( "string15", new FastScalar( "Scalar15" ));

        FastHash hash2 = new FastHash( aMap );
        return hash2;
    }

    /**
     * Performs the serialization test.
     */
    protected void runTest() throws Exception {
        String aSerialFile = getTestcasePath() + "/serialize.tmp";

        // Step 1: create an object model to serialize
        SimpleHash modelRoot = new SimpleHash();

        // Scalar models
        modelRoot.put( "string1", new SimpleScalar( "Scalar1" ));
        modelRoot.put( "string2", new FastScalar( "Scalar2" ));
        modelRoot.put( "string3", "Scalar3" );

        // Boolean models
        modelRoot.put( "boolean1", new SimpleScalar( false ));
        modelRoot.put( "boolean2", new SimpleScalar( true ));
        modelRoot.put( "boolean3", FastBoolean.getInstance( false ));
        modelRoot.put( "boolean4", FastBoolean.getInstance( true ));
        modelRoot.put( "boolean5", false );
        modelRoot.put( "boolean6", true );

        modelRoot.put( "number1", new FastNumber( 42 ));
        modelRoot.put( "number2", 42 );
        modelRoot.put( "number3", new SimpleNumber( 45 ));
        modelRoot.put( "number4", new SimpleNumber());


        // List models
        SimpleList list1 = makeList1();
        modelRoot.put( "list1", list1 );

        FastList list2 = makeList2();
        modelRoot.put( "list2", list2 );

        FastListRange list3 = new FastListRange( 10, 20 );
        modelRoot.put( "list3", list3 );

        // Hash models
        SimpleHash hash1 = makeHash1();
        modelRoot.put( "hash1", hash1 );

        FastHash hash2 = makeHash2();
        modelRoot.put( "hash2", hash2 );

        // Step 2: serialize the model
        FileOutputStream fo = new FileOutputStream(aSerialFile);
        ObjectOutputStream so = new ObjectOutputStream(fo);
        so.writeObject(modelRoot);
        so.flush();
        so.close();
        fo.close();

        // Step 3: re-read the serialized objects
        TemplateModelRoot newModel;
        FileInputStream fi = new FileInputStream(aSerialFile);
        ObjectInputStream si = new ObjectInputStream(fi);
        newModel = (TemplateModelRoot) si.readObject();
        si.close();
        fi.close();

        // Step 4: assert that the model is correct
        TemplateScalarModel stringModel = (TemplateScalarModel)newModel.get( "string1" );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        String string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar1" );
        TemplateModel compareModel = new SimpleScalar( "Scalar1" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "string2" );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar2" );
        compareModel = new FastScalar( "Scalar2" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "string3" );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar3" );
        compareModel = new FastScalar( "Scalar3" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean1" );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "false" );
        assertTrue( "Expected true", stringModel.isEmpty() );
        compareModel = new SimpleScalar( false );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean2" );
        assertTrue( "Expecting a SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "true" );
        assertTrue( "Expected false", ! stringModel.isEmpty() );
        compareModel = new SimpleScalar( true );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean3" );
        assertNotNull( stringModel );
        assertTrue( "Expecting a FastBoolean", stringModel instanceof FastBoolean );
        string = stringModel.getAsString();
        assertNull( "Value is: " + string, string );
        assertTrue( "Expected true", stringModel.isEmpty() );
        assertTrue( "Expected false", ! FastBoolean.getBoolean( stringModel ));
        compareModel = FastBoolean.getInstance( false );
        assertEquals( "FastBooleans should be equal", stringModel, compareModel );
        assertEquals( "FastBoolean hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean4" );
        assertNotNull( stringModel );
        assertTrue( "Expecting a FastBoolean", stringModel instanceof FastBoolean );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "true" );
        assertTrue( "Expected false", ! stringModel.isEmpty() );
        assertTrue( "Expected true", FastBoolean.getBoolean( stringModel ));
        compareModel = FastBoolean.getInstance( true );
        assertEquals( "FastBooleans should be equal", stringModel, compareModel );
        assertEquals( "FastBoolean hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean5" );
        assertNotNull( stringModel );
        assertTrue( "Expecting a FastBoolean", stringModel instanceof FastBoolean );
        string = stringModel.getAsString();
        assertNull( "Value is: " + string, string );
        assertTrue( "Expected true", stringModel.isEmpty() );
        assertTrue( "Expected false", ! FastBoolean.getBoolean( stringModel ));
        compareModel = FastBoolean.getInstance( false );
        assertEquals( "FastBooleans should be equal", stringModel, compareModel );
        assertEquals( "FastBoolean hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)newModel.get( "boolean6" );
        assertNotNull( stringModel );
        assertTrue( "Expecting a FastBoolean", stringModel instanceof FastBoolean );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "true" );
        assertTrue( "Expected false", ! stringModel.isEmpty() );
        assertTrue( "Expected true", FastBoolean.getBoolean( stringModel ));

        TemplateNumberModel numberModel = (TemplateNumberModel)newModel.get( "number1" );
        assertNotNull( numberModel );
        assertTrue( "Expecting a FastNumber", numberModel instanceof FastNumber );
        long number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 42, number );
        compareModel = new FastNumber( 42 );
        assertEquals( "FastNumbers should be equal", numberModel, compareModel );
        assertEquals( "FastNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        numberModel = (TemplateNumberModel)newModel.get( "number2" );
        assertNotNull( numberModel );
        assertTrue( "Expecting a FastNumber", numberModel instanceof FastNumber );
        number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, 42, number );
        assertEquals( "FastNumbers should be equal", numberModel, compareModel );
        assertEquals( "FastNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        numberModel = (TemplateNumberModel)newModel.get( "number3" );
        assertTrue( "Expecting a SimpleNumber", numberModel instanceof SimpleNumber );
        assertTrue( "Expecting non-empty value", numberModel.isEmpty() == false );
        number = numberModel.getAsNumber();
        assertEquals( "Value is: " + number, number, 45 );
        compareModel = new SimpleNumber( 45 );
        assertEquals( "SimpleNumbers should be equal", numberModel, compareModel );
        assertEquals( "SimpleNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        numberModel = (TemplateNumberModel)newModel.get( "number4" );
        assertTrue( "Expecting a SimpleNumber", numberModel instanceof SimpleNumber );
        assertTrue( "Expecting empty value", numberModel.isEmpty() );
        compareModel = new SimpleNumber();
        assertEquals( "SimpleNumbers should be equal", numberModel, compareModel );
        assertEquals( "SimpleNumber hash codes should match", numberModel.hashCode(), compareModel.hashCode() );

        TemplateIndexedModel listModel = (SimpleList)newModel.get( "list1" );
        assertNotNull( listModel );
        assertTrue( "Instance not a SimpleList", listModel instanceof SimpleList );
        compareModel = makeList1();
        assertEquals( "SimpleLists should be equal", listModel, compareModel );
        assertEquals( "SimpleList hash codes should match", listModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)listModel.getAtIndex( 0 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar4" );
        compareModel = new FastScalar( "Scalar4" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)listModel.getAtIndex( 1 );
        assertTrue( "Expecting a FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar5" );
        compareModel = new FastScalar( "Scalar5" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        listModel = (FastList)newModel.get( "list2" );
        assertNotNull( listModel );
        assertTrue( "Instance not a FastList", listModel instanceof FastList );
        compareModel = makeList2();
        assertEquals( "FastLists should be equal", listModel, compareModel );
        assertEquals( "FastList hash codes should match", listModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)listModel.getAtIndex( 0 );
        assertTrue( stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar6" );
        compareModel = new FastScalar( "Scalar6" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)listModel.getAtIndex( 1 );
        assertTrue( stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar7" );
        compareModel = new FastScalar( "Scalar7" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        listModel = (FastListRange)newModel.get( "list3" );
        assertNotNull( listModel );
        assertTrue( "Instance not a FastListRange", listModel instanceof FastListRange );

        for( int i = 0; i <= 10; i++ ) {
            numberModel = (TemplateNumberModel)listModel.getAtIndex( i );
            assertNotNull( numberModel );
            number = numberModel.getAsNumber();
            assertEquals( "Value is: " + number, i + 10, number );
        }

        compareModel = new FastListRange( 10, 20 );
        assertEquals( "FastListRanges should be equal", listModel, compareModel );
        assertEquals( "FastListRange hash codes should match", listModel.hashCode(), compareModel.hashCode() );

        TemplateHashModel hashModel = (TemplateHashModel)newModel.get( "hash1" );
        assertTrue( "Expecting SimpleHash", hashModel instanceof SimpleHash );
        compareModel = makeHash1();
        assertEquals( "SimpleHashes should be equal", hashModel, compareModel );
        assertEquals( "SimpleHash hash codes should match", hashModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string10" );
        assertTrue( "Expecting FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar10" );
        compareModel = new FastScalar( "Scalar10" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string11" );
        assertTrue( "Expecting FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar11" );
        compareModel = new FastScalar( "Scalar11" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string12" );
        assertTrue( "Expecting SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar12" );
        compareModel = new SimpleScalar( "Scalar12" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        hashModel = (TemplateHashModel)newModel.get( "hash2" );
        assertTrue( "Expecting FastHash", hashModel instanceof FastHash );
        compareModel = makeHash2();
        assertEquals( "FastHashes should be equal", hashModel, compareModel );
        assertEquals( "FastHash hash codes should match", hashModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string13" );
        assertTrue( "Expecting FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar13" );
        compareModel = new FastScalar( "Scalar13" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string14" );
        assertTrue( "Expecting SimpleScalar", stringModel instanceof SimpleScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar14" );
        compareModel = new SimpleScalar( "Scalar14" );
        assertEquals( "SimpleScalars should be equal", stringModel, compareModel );
        assertEquals( "SimpleScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

        stringModel = (TemplateScalarModel)hashModel.get( "string15" );
        assertTrue( "Expecting FastScalar", stringModel instanceof FastScalar );
        string = stringModel.getAsString();
        assertEquals( "Value is: " + string, string, "Scalar15" );
        compareModel = new FastScalar( "Scalar15" );
        assertEquals( "FastScalars should be equal", stringModel, compareModel );
        assertEquals( "FastScalar hash codes should match", stringModel.hashCode(), compareModel.hashCode() );

    }

    /**
     * Retrieve the root path of the FM-Classic distribution from a
     * properties file. This is used to set the root of the FM-Classic
     * cache, which in turn is used to find the example source code.
     *
     * @return the path to the root of the FM-Classic distribution
     * @throws IOException the <code>testcase.properties</code> file could
     * not be read
     */
    private String getTestcasePath() throws IOException {
        InputStream input = getClass().getResourceAsStream( "testcase.properties" );
        Properties properties = new Properties();

        properties.load( input );
        input.close();

        return (String)properties.get( "testcasePath" );
    }
}
